@@ -375,14 +375,14 @@ static int afm_update_seg(AFFILE *af, const char *name,
 }
 
 
-static int afm_read(AFFILE *af, unsigned char *buf, uint64 pos,int count)
+static int afm_read(AFFILE *af, unsigned char *buf, uint64 pos,size_t count)
 {
     struct afm_private *ap = AFM_PRIVATE(af);
     if(ap->sr_initialized==0 && afm_split_raw_setup(af)) return -1;
     return (*ap->sr->v->read)(ap->sr,buf,pos,count);
 }
 
-static int afm_write(AFFILE *af, unsigned char *buf, uint64 pos,int count)
+static int afm_write(AFFILE *af, unsigned char *buf, uint64 pos,size_t count)
 {
     struct afm_private *ap = AFM_PRIVATE(af);
     if(ap->sr_initialized==0 && afm_split_raw_setup(af)) return -1;
@@ -398,7 +398,7 @@ static int afm_write(AFFILE *af, unsigned char *buf, uint64 pos,int count)
 
 struct af_vnode vnode_afm = {
     AF_IDENTIFY_AFM,
-    AF_VNODE_TYPE_COMPOUND,
+    AF_VNODE_TYPE_COMPOUND|AF_VNODE_TYPE_RELIABLE|AF_VNODE_MAXSIZE_MULTIPLE,
     "AFM (AFF metadata with split raw file)",
     afm_identify_file,
     afm_open,
