@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  * 
- * $Date: 2007/04/25 21:46:05 $
+ * $Date: 2007/10/18 17:18:25 $
  */
 #ifndef _AUX_LIB_H
 #define _AUX_LIB_H
@@ -317,6 +317,12 @@ extern "C" {
 #define TSK_ERR_HDB_MAX		12
 
 
+// Return values
+typedef enum {
+    TSK_OK,     ///< Ok -- success
+    TSK_ERR,    ///< System error -- should abort
+    TSK_COR     ///< Data is corrupt, can still process another set of data
+} TSK_RETVAL_ENUM;
 
 /************* DATA BUF ******************/
     typedef struct TSK_DATA_BUF TSK_DATA_BUF;
@@ -553,11 +559,11 @@ typedef unsigned long int UINT4;
         UINT4 state[4];         /* state (ABCD) */
         UINT4 count[2];         /* number of bits, modulo 2^64 (lsb first) */
         unsigned char buffer[64];       /* input buffer */
-    } MD5_CTX;
+    } TSK_MD5_CTX;
 
-    void MD5Init(MD5_CTX *);
-    void MD5Update(MD5_CTX *, unsigned char *, unsigned int);
-    void MD5Final(unsigned char[16], MD5_CTX *);
+    void TSK_MD5_Init(TSK_MD5_CTX *);
+    void TSK_MD5_Update(TSK_MD5_CTX *, unsigned char *, unsigned int);
+    void TSK_MD5_Final(unsigned char[16], TSK_MD5_CTX *);
 
 
 
@@ -570,13 +576,13 @@ typedef unsigned long int UINT4;
         UINT4 countLo, countHi; /* 64-bit bit count */
         UINT4 data[16];         /* SHS data buffer */
         int Endianness;
-    } SHA_CTX;
+    } TSK_SHA_CTX;
 
 /* Message digest functions */
 
-    void SHAInit(SHA_CTX *);
-    void SHAUpdate(SHA_CTX *, BYTE * buffer, int count);
-    void SHAFinal(BYTE * output, SHA_CTX *);
+    void TSK_SHA_Init(TSK_SHA_CTX *);
+    void TSK_SHA_Update(TSK_SHA_CTX *, BYTE * buffer, int count);
+    void TSK_SHA_Final(BYTE * output, TSK_SHA_CTX *);
 
 
 
