@@ -8,6 +8,11 @@
 #include <unistd.h>
 #endif
 
+/**
+ * \file tsk_error.c
+ * Contains the error handling code and variables.
+ */
+
 
 /* Global variables that fit here as well as anywhere */
 char *progname = "unknown";
@@ -61,7 +66,7 @@ const char *tsk_err_img_str[TSK_ERR_IMG_MAX] = {
 
 
 const char *tsk_err_mm_str[TSK_ERR_MM_MAX] = {
-    "Cannot determine partiton type",
+    "Cannot determine partition type",
     "Unsupported partition type",
     "Error reading image file",
     "Invalid magic value",
@@ -105,9 +110,11 @@ const char *tsk_err_hdb_str[TSK_ERR_HDB_MAX] = {
 };
 
 
-/* return a string with the current error message 
- * return NULL if there is no error
- * this will reset the error number and messages
+/**
+ * Return the string with the current error message.  The string does not end with a 
+ * newline and it should not be freed.
+ *
+ * @returns String (in static buffer) with error message or NULL if there is no error
  */
 char *
 tsk_error_get()
@@ -176,13 +183,14 @@ tsk_error_get()
             " (%s)", tsk_errstr2);
         pidx = strlen(tsk_errstr_print);
     }
-
-    snprintf(&tsk_errstr_print[pidx], TSK_ERRSTR_PR_L - pidx, "\n");
-    tsk_error_reset();
     return (char *) &tsk_errstr_print[0];
 }
 
-/* Print the error message to hFile */
+/**
+ * Print the current error message to a file.
+ *
+ * @param hFile File to print message to
+ */
 void
 tsk_error_print(FILE * hFile)
 {
@@ -192,10 +200,10 @@ tsk_error_print(FILE * hFile)
 
     str = tsk_error_get();
     if (str != NULL) {
-        tsk_fprintf(hFile, "%s", str);
+        tsk_fprintf(hFile, "%s\n", str);
     }
     else {
-        tsk_fprintf(hFile, "Error Creating Error String");
+        tsk_fprintf(hFile, "Error creating Sleuth Kit error string (Errno: %d)\n", tsk_errno);
     }
 }
 
