@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2007/06/13 21:17:01 $
+** $Date: 2007/10/19 22:00:55 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2004-2005 Brian Carrier.  All rights reserved
@@ -14,7 +14,7 @@
      * Solaris 2.x. Build for large files when dealing with filesystems > 2GB.
      * With the 32-bit file model, needs pread() to access filesystems > 2GB.
      */
-#if defined(SUNOS5)
+#if defined(sun)
 #define SUPPORTED
 #define USE_LIBAFF
 #define	USE_LIBEWF
@@ -28,87 +28,20 @@
 #endif
 
 
-    /*
-     * FreeBSD can handle filesystems > 2GB.
-     */
-#if defined(FREEBSD2) || defined(FREEBSD3) || defined(FREEBSD4) || defined(FREEBSD5)
-#define SUPPORTED
-#define USE_LIBAFF
-#define	USE_LIBEWF
-#define HAVE_UNISTD
 
-/* FreeBSD 5 has inttypes and support for the printf macros */
-#if defined(FREEBSD4) || defined(FREEBSD5)
-#include <inttypes.h>
-#endif
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined (__OpenBSD__) || defined (__APPLE__) || \
+defined(linux) || defined (__linux__) || defined(__bsdi__)
 
-#endif                          /* FREEBSD */
-
-    /*
-     * BSD/OS can handle filesystems > 2GB.
-     */
-#if defined(BSDI2) || defined(BSDI3) || defined(BSDI4)
 #define SUPPORTED
 #define USE_LIBAFF
 #define	USE_LIBEWF
 #define HAVE_UNISTD
 
 #include <inttypes.h>
-#endif                          /* BSDI */
+#endif                          
 
 
-/*
- * NetBSD
- */
-#if defined(NETBSD16)
-#define SUPPORTED
-#define USE_LIBAFF
-#define	USE_LIBEWF
-#define HAVE_UNISTD
-
-#include <inttypes.h>
-#endif                          /* NETBSD */
-
-
-    /*
-     * OpenBSD looks like BSD/OS 3.x.
-     */
-#if defined(OPENBSD2) || defined (OPENBSD3)
-#define SUPPORTED
-#define USE_LIBAFF
-#define	USE_LIBEWF
-#define HAVE_UNISTD
-
-#include <inttypes.h>
-#endif
-
-
-
-#if defined(DARWIN)
-#define SUPPORTED
-#define USE_LIBAFF
-#define	USE_LIBEWF
-#define HAVE_UNISTD
-
-#include <inttypes.h>
-#endif                          /* DARWIN */
-
-
-    /*
-     * Linux 2.whatever. 
-     */
-#if defined(LINUX2)
-#define SUPPORTED
-#define USE_LIBAFF
-#define	USE_LIBEWF
-#define HAVE_UNISTD
-
-#include <inttypes.h>
-#endif                          /* LINUX */
-
-
-
-#if defined(CYGWIN)
+#if defined(__CYGWIN__)
 #define SUPPORTED
 #define USE_LIBAFF
 #define	USE_LIBEWF
@@ -132,6 +65,7 @@
 
 #endif                          /* INTERNIX */
 
+
 #if defined(_WIN32) || defined (__WIN32__)
 #define SUPPORTED
 #define TSK_WIN32
