@@ -2,7 +2,7 @@
 ** dcalc
 ** The Sleuth Kit 
 **
-** $Date: 2007/03/23 15:01:31 $
+** $Date: 2007/08/16 16:14:25 $
 **
 ** Calculates the corresponding block number between 'dls' and 'dd' images
 ** when given an 'dls' block number, it determines the block number it
@@ -169,6 +169,12 @@ MAIN(int argc, TSK_TCHAR ** argv)
         tsk_error_print(stderr);
         exit(1);
     }
+    if (imgoff >= img->size) {
+        tsk_fprintf(stderr,
+            "Sector offset supplied is larger than disk image (maximum: %"
+            PRIu64 ")\n", img->size / 512);
+        exit(1);
+    }
 
     if ((fs = tsk_fs_open(img, imgoff, fstype)) == NULL) {
         tsk_error_print(stderr);
