@@ -2,7 +2,7 @@
 ** dcat
 ** The  Sleuth Kit 
 **
-** $Date: 2007/04/05 16:01:55 $
+** $Date: 2007/08/16 16:14:25 $
 **
 ** Given an image , block number, and size, display the contents
 ** of the block to stdout.
@@ -174,6 +174,12 @@ MAIN(int argc, TSK_TCHAR ** argv)
             tsk_error_print(stderr);
             exit(1);
         }
+        if (imgoff >= img->size) {
+            tsk_fprintf(stderr,
+                "Sector offset supplied is larger than disk image (maximum: %"
+                PRIu64 ")\n", img->size / 512);
+            exit(1);
+        }
 
     }
     else {
@@ -194,6 +200,12 @@ MAIN(int argc, TSK_TCHAR ** argv)
                 tsk_error_print(stderr);
                 exit(1);
             }
+            if (imgoff >= img->size) {
+                tsk_fprintf(stderr,
+                    "Sector offset supplied is larger than disk image (maximum: %"
+                    PRIu64 ")\n", img->size / 512);
+                exit(1);
+            }
 
         }
         else {
@@ -217,6 +229,12 @@ MAIN(int argc, TSK_TCHAR ** argv)
                 tsk_error_print(stderr);
                 exit(1);
             }
+            if (imgoff >= img->size) {
+                tsk_fprintf(stderr,
+                    "Sector offset supplied is larger than disk image (maximum: %"
+                    PRIu64 ")\n", img->size / 512);
+                exit(1);
+            }
         }
     }
 
