@@ -2,7 +2,7 @@
 ** dcat
 ** The  Sleuth Kit 
 **
-** $Date: 2007/03/23 15:01:31 $
+** $Date: 2007/08/29 19:20:02 $
 **
 ** Given an image , block number, and size, display the contents
 ** of the block to stdout.
@@ -109,7 +109,7 @@ tsk_fs_dcat(TSK_FS_INFO * fs, uint8_t lclflags, DADDR_T addr,
         /* Read the block */
         cnt = tsk_fs_read_block(fs, buf, fs->block_size, addr + i);
         if (cnt != fs->block_size) {
-            if (cnt != -1) {
+            if (cnt >= 0) {
                 tsk_error_reset();
                 tsk_errno = TSK_ERR_FS_READ;
             }
