@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2007/05/17 19:32:28 $
+** $Date: 2007/11/05 18:23:14 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006-2007 Brian Carrier, Basis Technology.  All Rights reserved
@@ -79,7 +79,7 @@ ext2fs_group_load(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grp_num)
     cnt = tsk_fs_read_random(&ext2fs->fs_info, (char *) gd,
         sizeof(ext2fs_gd), offs);
     if (cnt != sizeof(ext2fs_gd)) {
-        if (cnt != -1) {
+        if (cnt >= 0) {
             tsk_error_reset();
             tsk_errno = TSK_ERR_FS_READ;
         }
@@ -184,7 +184,7 @@ ext2fs_bmap_load(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grp_num)
             ext2fs->grp_buf->bg_block_bitmap));
 
     if (cnt != ext2fs->fs_info.block_size) {
-        if (cnt != -1) {
+        if (cnt >= 0) {
             tsk_error_reset();
             tsk_errno = TSK_ERR_FS_READ;
         }
@@ -253,7 +253,7 @@ ext2fs_imap_load(EXT2FS_INFO * ext2fs, EXT2_GRPNUM_T grp_num)
             ext2fs->grp_buf->bg_inode_bitmap));
 
     if (cnt != ext2fs->fs_info.block_size) {
-        if (cnt != -1) {
+        if (cnt >= 0) {
             tsk_error_reset();
             tsk_errno = TSK_ERR_FS_READ;
         }
@@ -335,7 +335,7 @@ ext2fs_dinode_load(EXT2FS_INFO * ext2fs, INUM_T inum)
 
     cnt = tsk_fs_read_random(fs, (char *) dino, ext2fs->inode_size, addr);
     if (cnt != ext2fs->inode_size) {
-        if (cnt != -1) {
+        if (cnt >= 0) {
             tsk_error_reset();
             tsk_errno = TSK_ERR_FS_READ;
         }
@@ -502,7 +502,7 @@ ext2fs_dinode_copy(EXT2FS_INFO * ext2fs, TSK_FS_INODE * fs_inode)
                     fs_inode->direct_addr[i]);
 
                 if (cnt != fs->block_size) {
-                    if (cnt != -1) {
+                    if (cnt >= 0) {
                         tsk_error_reset();
                         tsk_errno = TSK_ERR_FS_READ;
                     }
@@ -867,7 +867,7 @@ ext2fs_block_walk(TSK_FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk,
                     tsk_fs_read_block(fs, data_buf, fs->block_size, addr);
 
                 if (cnt != fs->block_size) {
-                    if (cnt != -1) {
+                    if (cnt >= 0) {
                         tsk_error_reset();
                         tsk_errno = TSK_ERR_FS_READ;
                     }
@@ -980,7 +980,7 @@ ext2fs_block_walk(TSK_FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk,
             cnt = tsk_fs_read_block(fs, data_buf, fs->block_size, addr);
 
             if (cnt != fs->block_size) {
-                if (cnt != -1) {
+                if (cnt >= 0) {
                     tsk_error_reset();
                     tsk_errno = TSK_ERR_FS_READ;
                 }
@@ -1074,7 +1074,7 @@ ext2fs_file_walk_direct(TSK_FS_INFO * fs, TSK_DATA_BUF * buf[],
             SSIZE_T cnt;
             cnt = tsk_fs_read_block(fs, buf[0], rounded_size, addr);
             if (cnt != (SSIZE_T) rounded_size) {
-                if (cnt != -1) {
+                if (cnt >= 0) {
                     tsk_error_reset();
                     tsk_errno = TSK_ERR_FS_READ;
                 }
@@ -1141,7 +1141,7 @@ ext2fs_file_walk_indir(TSK_FS_INFO * fs, TSK_DATA_BUF * buf[],
         SSIZE_T cnt;
         cnt = tsk_fs_read_block(fs, buf[level], buf[level]->size, addr);
         if (cnt != buf[level]->size) {
-            if (cnt != -1) {
+            if (cnt >= 0) {
                 tsk_error_reset();
                 tsk_errno = TSK_ERR_FS_READ;
             }
@@ -1208,7 +1208,9 @@ ext2fs_file_walk_indir(TSK_FS_INFO * fs, TSK_DATA_BUF * buf[],
 }
 
 
-/*      
+
+/**
+ * Calls a callback function with the contents of each block in a file. 
  * flag values: TSK_FS_FILE_FLAG_NOSPARSE, TSK_FS_FILE_FLAG_AONLY, TSK_FS_FILE_FLAG_SLACK
  * TSK_FS_FILE_FLAG_META
  *
@@ -1217,11 +1219,15 @@ ext2fs_file_walk_indir(TSK_FS_INFO * fs, TSK_DATA_BUF * buf[],
  *
  * The action will use the flags: TSK_FS_BLOCK_FLAG_CONT, TSK_FS_BLOCK_FLAG_META
  * -- @@@ Currently do not do _ALLOC and _UNALLOC
- *  
- * The type and id fields are ignored with TSK_FS_INFO_TYPE_EXT_2
- *
- * return 1 on error and 0 on success
  *
+ * @param fs File system file is located in
+ * @param inode File to read and analyze
+ * @param type Attribute type to read and analyze (does not apply to Ext2)
+ * @param id Attribute id to read and analyze (does not apply to Ext2)
+ * @param flags Flags to use while reading
+ * @param action Callback function that is called for each block
+ * @param ptr Pointer to data that is passed to the callback
+ * @returns 1 on error and 0 on success
  */
 uint8_t
 ext2fs_file_walk(TSK_FS_INFO * fs, TSK_FS_INODE * inode, uint32_t type,
@@ -1337,8 +1343,14 @@ ext2fs_fscheck(TSK_FS_INFO * fs, FILE * hFile)
 }
 
 
-/* Return 0 on success and 1 on error */
-
+/**
+ * Print details about the file system to a file handle. 
+ *
+ * @param fs File system to print details on
+ * @param hFile File handle to print text to
+ * 
+ * @returns 1 on error and 0 on success
+ */
 static uint8_t
 ext2fs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
 {
@@ -1534,6 +1546,11 @@ ext2fs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
     tsk_fprintf(hFile, "Block Range: %" PRIuDADDR " - %" PRIuDADDR "\n",
         fs->first_block, fs->last_block);
 
+    if (fs->last_block != fs->last_block_act)
+        tsk_fprintf(hFile,
+            "Total Range in Image: %" PRIuDADDR " - %" PRIuDADDR "\n",
+            fs->first_block, fs->last_block_act);
+
     tsk_fprintf(hFile, "Block Size: %u\n", fs->block_size);
 
     if (tsk_getu32(fs->endian, sb->s_first_data_block))
@@ -1817,7 +1834,17 @@ print_addr_act(TSK_FS_INFO * fs, DADDR_T addr, char *buf,
 }
 
 
-/* Return 1 on error and 0 on success */
+/**
+ * Print details on a specific file to a file handle. 
+ *
+ * @param fs File system file is located in
+ * @param hFile File handle to print text to
+ * @param inum Address of file in file system
+ * @param numblock The number of blocks in file to force print (can go beyond file size)
+ * @param sec_skew Clock skew in seconds to also print times in
+ * 
+ * @returns 1 on error and 0 on success
+ */
 static uint8_t
 ext2fs_istat(TSK_FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
     int32_t sec_skew)
@@ -1934,7 +1961,7 @@ ext2fs_istat(TSK_FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
                 ext2fs->dino_buf->i_file_acl));
 
         if (cnt != fs->block_size) {
-            if (cnt != -1) {
+            if (cnt >= 0) {
                 tsk_error_reset();
                 tsk_errno = TSK_ERR_FS_READ;
             }
@@ -2230,11 +2257,15 @@ ext2fs_close(TSK_FS_INFO * fs)
     free(ext2fs);
 }
 
-/* ext2fs_open - open an ext2fs file system 
+/**
+ * Open part of a disk image as a Ext2/3 file system. 
  *
- * Return NULL on error (or not an Ext2 file system)
- * */
-
+ * @param img_info Disk image to analyze
+ * @param offset Byte offset where file system starts
+ * @param ftype Specific type of file system
+ * @param test NOT USED
+ * @returns NULL on error or if data is not an Ext2/3 file system
+ */
 TSK_FS_INFO *
 ext2fs_open(TSK_IMG_INFO * img_info, SSIZE_T offset,
     TSK_FS_INFO_TYPE_ENUM ftype, uint8_t test)
@@ -2278,7 +2309,7 @@ ext2fs_open(TSK_IMG_INFO * img_info, SSIZE_T offset,
         tsk_fs_read_random(fs, (char *) ext2fs->fs, len,
         (OFF_T) EXT2FS_SBOFF);
     if (cnt != len) {
-        if (cnt != -1) {
+        if (cnt >= 0) {
             tsk_error_reset();
             tsk_errno = TSK_ERR_FS_READ;
         }
@@ -2362,11 +2393,10 @@ ext2fs_open(TSK_IMG_INFO * img_info, SSIZE_T offset,
     fs->dev_bsize = EXT2FS_DEV_BSIZE;
     fs->block_count = tsk_getu32(fs->endian, ext2fs->fs->s_blocks_count);
     fs->first_block = 0;
-    fs->last_block = fs->block_count - 1;
+    fs->last_block_act = fs->last_block = fs->block_count - 1;
     ext2fs->first_data_block =
         tsk_getu32(fs->endian, ext2fs->fs->s_first_data_block);
 
-
     if (tsk_getu32(fs->endian, ext2fs->fs->s_log_block_size) !=
         tsk_getu32(fs->endian, ext2fs->fs->s_log_frag_size)) {
         free(ext2fs->fs);
@@ -2382,6 +2412,11 @@ ext2fs_open(TSK_IMG_INFO * img_info, SSIZE_T offset,
         EXT2FS_MIN_BLOCK_SIZE << tsk_getu32(fs->endian,
         ext2fs->fs->s_log_block_size);
 
+    // determine the last block we have in this image
+    if ((img_info->size - offset) / fs->block_size < fs->last_block)
+        fs->last_block_act =
+            (img_info->size - offset) / fs->block_size - 1;
+
     /* The group descriptors are located in the block following the 
      * super block
      */
