@@ -2,7 +2,7 @@
 ** ext2fs_dent
 ** The Sleuth Kit 
 **
-** $Date: 2007/05/17 19:32:28 $
+** $Date: 2007/11/05 18:23:14 $
 **
 ** File name layer support for an TSK_FS_INFO_TYPE_EXT_2 image
 **
@@ -223,10 +223,8 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
          ** Check if we may have a valid directory entry.  If we don't,
          ** then increment to the next word and try again.  
          */
-        if ((inode > fs->last_inum) ||
-            (inode < 0) ||
-            (namelen > EXT2FS_MAXNAMLEN) ||
-            (namelen <= 0) ||
+        if ((inode > fs->last_inum) ||  // inode is unsigned
+            (namelen > EXT2FS_MAXNAMLEN) || (namelen == 0) ||   // namelen is unsigned
             (reclen < minreclen) || (reclen % 4) || (idx + reclen > len)) {
 
             minreclen = 4;
@@ -239,8 +237,7 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
          * sure that it also ends in the unalloc space */
         if ((dellen) && (dellen < minreclen)) {
             minreclen = 4;
-            if (dellen > 0)
-                dellen -= 4;
+            dellen -= 4;
             continue;
         }
 
@@ -357,19 +354,16 @@ ext2fs_dent_parse_block(EXT2FS_INFO * ext2fs, EXT2FS_DINFO * dinfo,
 }                               /* end ext2fs_dent_parse_block() */
 
 
-/* 
-** The main function to do directory entry walking
-**
-** action is called for each entry with flags set to TSK_FS_DENT_FLAG_ALLOC for
-** active entries
-**
-** this calls ext2fs_dent_parse_block to do the actual analysis
-**
-** Use the following flags: TSK_FS_DENT_FLAG_ALLOC, TSK_FS_DENT_FLAG_UNALLOC, 
-** TSK_FS_DENT_FLAG_RECURSE
-**
-** returns 0 on success and 1 on error
-*/
+/**
+ * Process the contents of a directory and pass each file name to a callback function.
+ *
+ * @param fs File system to analyze
+ * @param inode Metadata address of directory to analyze
+ * @param flags Flags used during analysis
+ * @param action Callback function that is called for each file name
+ * @param ptr Pointer to data that is passed to callback
+ * @returns 1 on error and 0 on success
+ */
 uint8_t
 ext2fs_dent_walk(TSK_FS_INFO * fs, INUM_T inode,
     TSK_FS_DENT_FLAG_ENUM flags, TSK_FS_DENT_TYPE_WALK_CB action,
