@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2007/04/04 20:06:58 $
+** $Date: 2007/11/30 16:44:20 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -12,6 +12,10 @@
 ** This software is distributed under the Common Public License 1.0
 **
 */
+/** \file fatfs.h
+ * Contains the structures and function APIs for FATFS file system support.
+ */
+
 
 #ifndef _FATFS_H
 #define _FATFS_H
@@ -199,6 +203,10 @@ extern "C" {
 	  (((c) >= 0x5b) && ((c) <= 0x5d)) || \
 	  ((c) == 0x7c)) == 0)
 
+// extensions are to be ascii / latin
+#define FATFS_IS_83_EXT(c)		\
+    (FATFS_IS_83_NAME((c)) && ((c) < 0x7f))
+
 
 
 /* flags for attributes field */
@@ -220,7 +228,7 @@ extern "C" {
 #define FATFS_SEC_MASK		0x1f    /* number of seconds div by 2 */
 #define FATFS_SEC_SHIFT		0
 #define FATFS_SEC_MIN		0
-#define FATFS_SEC_MAX		29
+#define FATFS_SEC_MAX		30
 #define FATFS_MIN_MASK		0x7e0   /* number of minutes 0-59 */
 #define FATFS_MIN_SHIFT		5
 #define FATFS_MIN_MIN		0
@@ -327,7 +335,6 @@ extern "C" {
         uint32_t sectperfat;    /* sectors per fat table */
         uint16_t numroot;       /* number of 32-byte dentries in root dir */
         uint32_t mask;          /* the mask to use for the sectors */
-
     } FATFS_INFO;
 
 
@@ -337,7 +344,7 @@ extern "C" {
         TSK_FS_DENT_FLAG_ENUM, TSK_FS_DENT_TYPE_WALK_CB, void *);
     extern uint8_t fatfs_isdentry(FATFS_INFO *, fatfs_dentry *);
     extern uint8_t fatfs_make_root(FATFS_INFO *, TSK_FS_INODE *);
-    extern uint8_t fatfs_dinode_copy(FATFS_INFO *, TSK_FS_INODE *,
+    extern TSK_RETVAL_ENUM fatfs_dinode_copy(FATFS_INFO *, TSK_FS_INODE *,
         fatfs_dentry *, DADDR_T, INUM_T);
 
 #ifdef __cplusplus
