@@ -2,7 +2,7 @@
 ** fatfs_dent
 ** The Sleuth Kit 
 **
-** $Date: 2007/04/05 16:01:56 $
+** $Date: 2007/11/05 18:23:14 $
 **
 ** Human interface Layer support for the FAT file system
 **
@@ -92,6 +92,11 @@ typedef struct {
      */
     uint8_t recdel;
 
+    /* Set to 1 when we are collecting inode allocation information. 
+     * The info is used by inode_walk when looking for ORPHAN files.
+     */
+    uint8_t save_inum_named;
+
 } FATFS_DINFO;
 
 
@@ -490,7 +495,7 @@ fatfs_dent_parse_buf(FATFS_INFO * fatfs, FATFS_DINFO * dinfo,
             /* Get inode */
             fs_dent->inode = inode;
             if (fatfs_dinode_copy(fatfs, fs_dent->fsi, dir, addrs[sidx],
-                    inode)) {
+                    inode) != TSK_OK) {
                 if (tsk_verbose)
                     tsk_fprintf(stderr,
                         "fatfs_dent_parse: could not copy inode -- ignoring\n");
@@ -552,14 +557,35 @@ fatfs_dent_parse_buf(FATFS_INFO * fatfs, FATFS_DINFO * dinfo,
 
                 if (retval == TSK_WALK_STOP) {
                     tsk_fs_dent_free(fs_dent);
+
+                    /* free the list -- the main API has no way
+                     * of knowing that we stopped early w/out error.
+                     */
+                    if (dinfo->save_inum_named) {
+                        tsk_list_free(fs->list_inum_named);
+                        fs->list_inum_named = NULL;
+                        dinfo->save_inum_named = 0;
+                    }
                     return 1;
                 }
                 else if (retval == TSK_WALK_ERROR) {
                     tsk_fs_dent_free(fs_dent);
                     return -1;
                 }
-            }
 
+                // save the inode info -- if the setup is right
+                if ((dinfo->save_inum_named) && (fs_dent->fsi)
+                    && (fs_dent->fsi->flags & TSK_FS_INODE_FLAG_UNALLOC)) {
+                    if (tsk_list_add(&fs->list_inum_named,
+                            fs_dent->fsi->addr)) {
+
+                        // if there is an error, then clear the list
+                        tsk_list_free(fs->list_inum_named);
+                        fs->list_inum_named = NULL;
+                        dinfo->save_inum_named = 0;
+                    }
+                }
+            }
 
             /* if we have a directory and need to recurse then do it */
             if (((fs_dent->fsi->mode & TSK_FS_INODE_MODE_FMT) ==
@@ -708,16 +734,16 @@ fatfs_dent_action(TSK_FS_INFO * fs, DADDR_T addr, char *buf,
 }
 
 
-/** 
- * The main (external) function to do directory entry walking
+
+/**
+ * Process the contents of a directory and pass each file name to a callback function.
  *
- * @param fs File system state structure
- * @param inode Root directory inode to start walking on
- * @param flags flags to use while walking (TSK_FS_DENT_FLAG_ALLOC, TSK_FS_DENT_FLAG_UNALLOC, and TSK_FS_DENT_FLAG_RECURSE)
- # @param action Callback to call for each directory entry.
- * @param ptr Pointer to data that should be passed to callback. 
- *
- * @return 1 on error and 0 on success.
+ * @param fs File system to analyze
+ * @param inode Metadata address of directory to analyze
+ * @param flags Flags used during analysis
+ * @param action Callback function that is called for each file name
+ * @param ptr Pointer to data that is passed to callback
+ * @returns 1 on error and 0 on success
  */
 uint8_t
 fatfs_dent_walk(TSK_FS_INFO * fs, INUM_T inode,
@@ -738,10 +764,29 @@ fatfs_dent_walk(TSK_FS_INFO * fs, INUM_T inode,
     }
 
     memset(&dinfo, 0, sizeof(FATFS_DINFO));
+
+    /* if the flags are right, we can collect info that may be needed
+     * for an orphan walk.  If the walk fails or stops, the code that
+     * calls the action will clear this stuff. 
+     */
+    if ((fs->list_inum_named == NULL) && (inode == fs->root_inum) &&
+        (flags & (TSK_FS_DENT_FLAG_ALLOC | TSK_FS_DENT_FLAG_UNALLOC |
+                TSK_FS_DENT_FLAG_RECURSE))) {
+        dinfo.save_inum_named = 1;
+    }
+
     retval =
         fatfs_dent_walk_lcl(fs, &dinfo, &list_seen, inode, flags, action,
         ptr);
 
+    /* If there was an error, then we stopped early and we should get
+     * rid of the partial list we were making.
+     */
+    if ((retval == 1) && (dinfo.save_inum_named == 1)) {
+        tsk_list_free(fs->list_inum_named);
+        fs->list_inum_named = NULL;
+    }
+
     tsk_list_free(list_seen);
     list_seen = NULL;
     return retval;
