@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2007/05/17 19:32:28 $
+** $Date: 2007/11/05 18:23:14 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006-2007 Brian Carrier, Basis Technology.  All Rights reserved
@@ -73,7 +73,7 @@ ffs_group_load(FFS_INFO * ffs, FFS_GRPNUM_T grp_num)
         cnt =
             tsk_fs_read_block(fs, ffs->grp_buf, ffs->grp_buf->size, addr);
         if (cnt != ffs->grp_buf->size) {
-            if (cnt != -1) {
+            if (cnt >= 0) {
                 tsk_error_reset();
                 tsk_errno = TSK_ERR_FS_READ;
             }
@@ -181,7 +181,7 @@ ffs_dinode_load(FFS_INFO * ffs, INUM_T inum)
                 cnt = tsk_fs_read_block
                     (fs, ffs->itbl_buf, ffs->itbl_buf->size, addr);
                 if (cnt != ffs->itbl_buf->size) {
-                    if (cnt != -1) {
+                    if (cnt >= 0) {
                         tsk_error_reset();
                         tsk_errno = TSK_ERR_FS_READ;
                     }
@@ -215,7 +215,7 @@ ffs_dinode_load(FFS_INFO * ffs, INUM_T inum)
                 tsk_fs_read_block(fs, ffs->itbl_buf, ffs->itbl_buf->size,
                 addr);
             if (cnt != ffs->itbl_buf->size) {
-                if (cnt != -1) {
+                if (cnt >= 0) {
                     tsk_error_reset();
                     tsk_errno = TSK_ERR_FS_READ;
                 }
@@ -362,7 +362,7 @@ ffs_dinode_copy(FFS_INFO * ffs, TSK_FS_INODE * fs_inode)
                     cnt = tsk_fs_read_block(fs, data_buf, fs->block_size,
                         fs_inode->direct_addr[i]);
                     if (cnt != fs->block_size) {
-                        if (cnt != -1) {
+                        if (cnt >= 0) {
                             tsk_error_reset();
                             tsk_errno = TSK_ERR_FS_READ;
                         }
@@ -487,7 +487,7 @@ ffs_dinode_copy(FFS_INFO * ffs, TSK_FS_INODE * fs_inode)
                     cnt = tsk_fs_read_block(fs, data_buf, fs->block_size,
                         fs_inode->direct_addr[i]);
                     if (cnt != fs->block_size) {
-                        if (cnt != -1) {
+                        if (cnt >= 0) {
                             tsk_error_reset();
                             tsk_errno = TSK_ERR_FS_READ;
                         }
@@ -610,7 +610,7 @@ ffs_dinode_copy(FFS_INFO * ffs, TSK_FS_INODE * fs_inode)
                     cnt = tsk_fs_read_block(fs, data_buf, fs->block_size,
                         fs_inode->direct_addr[i]);
                     if (cnt != fs->block_size) {
-                        if (cnt != -1) {
+                        if (cnt >= 0) {
                             tsk_error_reset();
                             tsk_errno = TSK_ERR_FS_READ;
                         }
@@ -1104,7 +1104,7 @@ ffs_block_walk(TSK_FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk,
                         tsk_fs_read_block(fs, data_buf,
                         fs->block_size * frags, addr);
                     if (cnt != fs->block_size * frags) {
-                        if (cnt != -1) {
+                        if (cnt >= 0) {
                             tsk_error_reset();
                             tsk_errno = TSK_ERR_FS_READ;
                         }
@@ -1213,7 +1213,7 @@ ffs_file_walk_direct(TSK_FS_INFO * fs, TSK_DATA_BUF * buf,
                 tsk_fs_read_block(fs, buf, roundup(read_count,
                     FFS_DEV_BSIZE), addr);
             if (cnt != (SSIZE_T) roundup(read_count, FFS_DEV_BSIZE)) {
-                if (cnt != -1) {
+                if (cnt >= 0) {
                     tsk_error_reset();
                     tsk_errno = TSK_ERR_FS_READ;
                 }
@@ -1273,7 +1273,7 @@ ffs_file_walk_indir(TSK_FS_INFO * fs, TSK_DATA_BUF * buf[], OFF_T length,
         SSIZE_T cnt;
         cnt = tsk_fs_read_block(fs, buf[level], buf[level]->size, addr);
         if (cnt != buf[level]->size) {
-            if (cnt != -1) {
+            if (cnt >= 0) {
                 tsk_error_reset();
                 tsk_errno = TSK_ERR_FS_READ;
             }
@@ -1383,20 +1383,26 @@ ffs_file_walk_indir(TSK_FS_INFO * fs, TSK_DATA_BUF * buf[], OFF_T length,
     return (SSIZE_T) (length - todo_count);
 }
 
-/*  
- *  
- * flag values: FS_FLAG_FILE_SPARSE, TSK_FS_FILE_FLAG_AONLY, TSK_FS_FILE_FLAG_SLACK
- *  TSK_FS_FILE_FLAG_META
+
+/**
+ * Calls a callback function with the contents of each block in a file. 
+ * flag values: TSK_FS_FILE_FLAG_NOSPARSE, TSK_FS_FILE_FLAG_AONLY, TSK_FS_FILE_FLAG_SLACK
+ * TSK_FS_FILE_FLAG_META
  *
- * If TSK_FS_FILE_FLAG_RECOVER is set, then many error codes are set to _RECOVER
- * so that errors can be suppressed.  NO special recovery logic is used. 
+ * If TSK_FS_FILE_FLAG_RECOVER is set, then most error codes are set to
+ * _RECOVER.  No special recovery logic exists in this code. 
  *
- * Action uses: TSK_FS_BLOCK_FLAG_CONT, TSK_FS_BLOCK_FLAG_META
- * @@@ DATA_ALLOC and _UNALLOC are not implemented
- *  
- * The type and id fields are ignored with FFS
+ * The action will use the flags: TSK_FS_BLOCK_FLAG_CONT, TSK_FS_BLOCK_FLAG_META
+ * -- @@@ Currently do not do _ALLOC and _UNALLOC
  *
- * return 1 on error and 0 on success
+ * @param fs File system file is located in
+ * @param inode File to read and analyze
+ * @param type Attribute type to read and analyze (does not apply to FFS)
+ * @param id Attribute id to read and analyze (does not apply to FFS)
+ * @param flags Flags to use while reading
+ * @param action Callback function that is called for each block
+ * @param ptr Pointer to data that is passed to the callback
+ * @returns 1 on error and 0 on success
  */
 uint8_t
 ffs_file_walk(TSK_FS_INFO * fs, TSK_FS_INODE * inode, uint32_t type,
@@ -1513,8 +1519,13 @@ ffs_fscheck(TSK_FS_INFO * fs, FILE * hFile)
 }
 
 
-/*
- * return 1 on error and 0 on success
+/**
+ * Print details about the file system to a file handle. 
+ *
+ * @param fs File system to print details on
+ * @param hFile File handle to print text to
+ * 
+ * @returns 1 on error and 0 on success
  */
 static uint8_t
 ffs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
@@ -1619,6 +1630,11 @@ ffs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
     tsk_fprintf(hFile, "Fragment Range: %" PRIuDADDR " - %" PRIuDADDR "\n",
         fs->first_block, fs->last_block);
 
+    if (fs->last_block != fs->last_block_act)
+        tsk_fprintf(hFile,
+            "Total Range in Image: %" PRIuDADDR " - %" PRIuDADDR "\n",
+            fs->first_block, fs->last_block_act);
+
     tsk_fprintf(hFile, "Block Size: %u\n", ffs->ffsbsize_b);
     tsk_fprintf(hFile, "Fragment Size: %u\n", fs->block_size);
 
@@ -1664,7 +1680,7 @@ ffs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
                 (DADDR_T) tsk_getu32(fs->endian, sb1->cg_saddr));
 
             if (cnt != tsk_getu32(fs->endian, sb1->cg_ssize_b)) {
-                if (cnt != -1) {
+                if (cnt >= 0) {
                     tsk_error_reset();
                     tsk_errno = TSK_ERR_FS_READ;
                 }
@@ -1680,7 +1696,7 @@ ffs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
                     sb2->cg_ssize_b), (DADDR_T) tsk_getu64(fs->endian,
                     sb2->cg_saddr));
             if (cnt != tsk_getu32(fs->endian, sb2->cg_ssize_b)) {
-                if (cnt != -1) {
+                if (cnt >= 0) {
                     tsk_error_reset();
                     tsk_errno = TSK_ERR_FS_READ;
                 }
@@ -1885,7 +1901,16 @@ print_addr_act(TSK_FS_INFO * fs, DADDR_T addr, char *buf,
 
 
 
-/* return 1 on error and 0 on success
+/**
+ * Print details on a specific file to a file handle. 
+ *
+ * @param fs File system file is located in
+ * @param hFile File handle to print text to
+ * @param inum Address of file in file system
+ * @param numblock The number of blocks in file to force print (can go beyond file size)
+ * @param sec_skew Clock skew in seconds to also print times in
+ * 
+ * @returns 1 on error and 0 on success
  */
 static uint8_t
 ffs_istat(TSK_FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
@@ -1980,7 +2005,7 @@ ffs_istat(TSK_FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
                 cnt = tsk_fs_read_block(fs, data_buf, ffs->ffsbsize_b,
                     tsk_getu64(fs->endian, in->di_extb[0]));
                 if (cnt != ffs->ffsbsize_b) {
-                    if (cnt != -1) {
+                    if (cnt >= 0) {
                         tsk_error_reset();
                         tsk_errno = TSK_ERR_FS_READ;
                     }
@@ -2020,7 +2045,7 @@ ffs_istat(TSK_FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
                 cnt = tsk_fs_read_block(fs, data_buf, ffs->ffsbsize_b,
                     tsk_getu64(fs->endian, in->di_extb[1]));
                 if (cnt != ffs->ffsbsize_b) {
-                    if (cnt != -1) {
+                    if (cnt >= 0) {
                         tsk_error_reset();
                         tsk_errno = TSK_ERR_FS_FUNC;
                     }
@@ -2157,8 +2182,14 @@ ffs_close(TSK_FS_INFO * fs)
     free(ffs);
 }
 
-/* ffs_open - open a fast file system */
-
+/**
+ * Open part of a disk image as a FFS/UFS file system. 
+ *
+ * @param img_info Disk image to analyze
+ * @param offset Byte offset where file system starts
+ * @param ftype Specific type of file system
+ * @returns NULL on error or if data is not a FFS file system
+ */
 TSK_FS_INFO *
 ffs_open(TSK_IMG_INFO * img_info, SSIZE_T offset,
     TSK_FS_INFO_TYPE_ENUM ftype)
@@ -2208,7 +2239,7 @@ ffs_open(TSK_IMG_INFO * img_info, SSIZE_T offset,
     cnt = tsk_fs_read_random
         (fs, (char *) ffs->fs.sb2, sizeof(ffs_sb2), (OFF_T) UFS2_SBOFF);
     if (cnt != sizeof(ffs_sb2)) {
-        if (cnt != -1) {
+        if (cnt >= 0) {
             tsk_error_reset();
             tsk_errno = TSK_ERR_FS_READ;
         }
@@ -2225,7 +2256,7 @@ ffs_open(TSK_IMG_INFO * img_info, SSIZE_T offset,
             (fs, (char *) ffs->fs.sb2, sizeof(ffs_sb2),
             (OFF_T) UFS2_SBOFF2);
         if (cnt != sizeof(ffs_sb2)) {
-            if (cnt != -1) {
+            if (cnt >= 0) {
                 tsk_error_reset();
                 tsk_errno = TSK_ERR_FS_READ;
             }
@@ -2242,7 +2273,7 @@ ffs_open(TSK_IMG_INFO * img_info, SSIZE_T offset,
             cnt = tsk_fs_read_random
                 (fs, (char *) ffs->fs.sb1, len, (OFF_T) UFS1_SBOFF);
             if (cnt != len) {
-                if (cnt != -1) {
+                if (cnt >= 0) {
                     tsk_error_reset();
                     tsk_errno = TSK_ERR_FS_READ;
                 }
@@ -2297,9 +2328,14 @@ ffs_open(TSK_IMG_INFO * img_info, SSIZE_T offset,
 
 
     fs->first_block = 0;
-    fs->last_block = fs->block_count - 1;
+    fs->last_block = fs->last_block_act = fs->block_count - 1;
     fs->dev_bsize = FFS_DEV_BSIZE;
 
+    // determine the last block we have in this image
+    if ((img_info->size - offset) / fs->block_size < fs->last_block)
+        fs->last_block_act =
+            (img_info->size - offset) / fs->block_size - 1;
+
     if ((fs->block_size % 512) || (ffs->ffsbsize_b % 512)) {
         free(ffs->fs.sb1);
         free(ffs);
