@@ -2,7 +2,7 @@
 ** fs_data
 ** The Sleuth Kit 
 **
-** $Date: 2007/04/05 16:01:58 $
+** $Date: 2007/11/29 02:21:44 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006-2007 Brian Carrier, Basis Technology.  All Rights reserved
@@ -59,6 +59,7 @@ tsk_fs_data_alloc(TSK_FS_DATA_FLAG_ENUM type)
     fs_data->size = 0;
     fs_data->flags = 0;
     fs_data->run = NULL;
+    fs_data->run_end = NULL;
     fs_data->type = 0;
     fs_data->next = NULL;
     fs_data->compsize = 0;
@@ -172,6 +173,7 @@ tsk_fs_data_clear_list(TSK_FS_DATA * fs_data_head)
         if (fs_data_head->run) {
             tsk_fs_data_run_free(fs_data_head->run);
             fs_data_head->run = NULL;
+            fs_data_head->run_end = NULL;
             fs_data_head->allocsize = 0;
         }
         fs_data_head = fs_data_head->next;
@@ -428,10 +430,8 @@ tsk_fs_data_put_str(TSK_FS_DATA * fs_data_head, char *name, uint32_t type,
  * so we use "filler" TSK_FS_DATA_RUN structures during the process
  *
  * @param fs_data_head The head of the list of attributes (or NULL if list is empty)
- * @param start_vcn The virtual cluster number of the start of this run
- * in the attribute.
  * @param runlen The total number of clusters in this set of runs.
- * @param run The set of runs to add 
+ * @param data_run_new The set of runs to add.   This can be NULL only if it is the only run in the attribute.  We use this special case for $Bad, but it should change.
  * @param name Name of the attribute (in case it needs to be created)
  * @param type Type of attribute to add run to
  * @param id Id of attribute to add run to
@@ -443,17 +443,17 @@ tsk_fs_data_put_str(TSK_FS_DATA * fs_data_head, char *name, uint32_t type,
  */
 TSK_FS_DATA *
 tsk_fs_data_put_run(TSK_FS_DATA * fs_data_head,
-    DADDR_T start_vcn, OFF_T runlen, TSK_FS_DATA_RUN * run,
+    OFF_T runlen, TSK_FS_DATA_RUN * data_run_new,
     char *name, uint32_t type, uint16_t id, OFF_T size,
     TSK_FS_DATA_FLAG_ENUM flags, uint32_t compsize)
 {
     TSK_FS_DATA *fs_data = NULL;
-    TSK_FS_DATA_RUN *data_run, *data_run_prev;
-    DADDR_T cur_vcn;
+    TSK_FS_DATA_RUN *data_run_cur, *data_run_prev;
 
     tsk_error_reset();
 
     /* First thing is to find the existing data attribute */
+    fs_data = NULL;
     if (fs_data_head)
         fs_data = tsk_fs_data_lookup(fs_data_head, type, id);
 
@@ -487,8 +487,19 @@ tsk_fs_data_put_run(TSK_FS_DATA * fs_data_head,
         }
 
 
-        /* Add the run to the attribute.
-         *
+        /* Add the data_run_new to the attribute. */
+
+        /* We support the ODD case where the run is NULL.  In this case, 
+         * we set the attribute size info, but set everything else to NULL.
+         */
+        if (data_run_new == NULL) {
+            fs_data->allocsize = runlen;
+            fs_data->run = NULL;
+            fs_data->run_end = NULL;
+            return fs_data_head;
+        }
+
+        /*
          * If this is not in the begining, then we need to make a filler 
          * to account for the cluster numbers we haven't seen yet
          *
@@ -496,41 +507,69 @@ tsk_fs_data_put_run(TSK_FS_DATA * fs_data_head,
          * is not a base entry and it is referenced in an $ATTR_LIST
          *
          * The $DATA attribute in the non-base have a non-zero
-         * start_vcn.  
+         * data_run_new->offset.  
          */
-        if (start_vcn != 0) {
+        if (data_run_new->offset != 0) {
             TSK_FS_DATA_RUN *fill_run = tsk_fs_data_run_alloc();
             fill_run->flags = TSK_FS_DATA_RUN_FLAG_FILLER;
+            fill_run->offset = 0;
             fill_run->addr = 0;
-            fill_run->len = start_vcn;
-            fill_run->next = run;
-            run = fill_run;
+            fill_run->len = data_run_new->offset;
+            fill_run->next = data_run_new;
+            data_run_new = fill_run;
         }
 
         fs_data->allocsize = runlen;
-        fs_data->run = run;
+        fs_data->run = data_run_new;
+
+        // update the pointer to the end of the list
+        fs_data->run_end = data_run_new;
+        while (fs_data->run_end->next)
+            fs_data->run_end = fs_data->run_end->next;
 
         return fs_data_head;
     }
 
+    // we only support the case of a null run if it is the only run...
+    if (data_run_new == NULL) {
+        tsk_error_reset();
+        tsk_errno = TSK_ERR_FS_ARG;
+        snprintf(tsk_errstr, TSK_ERRSTR_L,
+            "tsk_fs_data_put_run: Error, NULL run added to existing attribute");
+        return NULL;
+    }
+
     /* 
      * An attribute of this type and id already exist, 
      * so we will either add to 
      * the end of it or replace a filler object with real data
      */
 
-    data_run = fs_data->run;
+    /* First thing, is to check if we can just add it to the end */
+    if ((fs_data->run_end)
+        && (fs_data->run_end->offset + fs_data->run_end->len ==
+            data_run_new->offset)) {
+        fs_data->run_end->next = data_run_new;
+        fs_data->allocsize += runlen;
+
+        // update the pointer to the end of the list
+        while (fs_data->run_end->next)
+            fs_data->run_end = fs_data->run_end->next;
+
+        /* return head of fs_data list */
+        return fs_data_head;
+    }
+
+    data_run_cur = fs_data->run;
     data_run_prev = NULL;
-    cur_vcn = 0;
-    while (data_run) {
+    while (data_run_cur) {
 
         /* Do we replace this filler spot? */
-        if (data_run->flags & TSK_FS_DATA_RUN_FLAG_FILLER) {
+        if (data_run_cur->flags & TSK_FS_DATA_RUN_FLAG_FILLER) {
 
             /* This should never happen because we always add 
-             * the filler to start from VCN 0
-             */
-            if (cur_vcn > start_vcn) {
+             * the filler to start from VCN 0 */
+            if (data_run_cur->offset > data_run_new->offset) {
                 tsk_error_reset();
                 tsk_errno = TSK_ERR_FS_ARG;
                 snprintf(tsk_errstr, TSK_ERRSTR_L,
@@ -538,20 +577,20 @@ tsk_fs_data_put_run(TSK_FS_DATA * fs_data_head,
                 return NULL;
             }
 
-            /* The current filler ends after where we need to 
-             * start, so it will be added here 
-             */
-            if (cur_vcn + data_run->len > start_vcn) {
+            /* Check if the new run starts inside of this filler. */
+            if (data_run_cur->offset + data_run_cur->len >
+                data_run_new->offset) {
                 TSK_FS_DATA_RUN *endrun;
 
                 /* if the new starts at the same as the filler, 
                  * replace the pointer */
-                if (cur_vcn == start_vcn) {
+                if (data_run_cur->offset == data_run_new->offset) {
                     if (data_run_prev)
-                        data_run_prev->next = run;
+                        data_run_prev->next = data_run_new;
                     else
-                        fs_data->run = run;
+                        fs_data->run = data_run_new;
                 }
+
                 /* The new run does not start at the begining of
                  * the filler, so make a new start filler
                  */
@@ -565,40 +604,44 @@ tsk_fs_data_put_run(TSK_FS_DATA * fs_data_head,
                     else
                         fs_data->run = newfill;
 
-                    newfill->next = run;
-                    newfill->len = start_vcn - cur_vcn;
+                    newfill->next = data_run_new;
+                    newfill->len =
+                        data_run_new->offset - data_run_cur->offset;
+                    newfill->offset = data_run_cur->offset;
                     newfill->flags = TSK_FS_DATA_RUN_FLAG_FILLER;
 
-                    data_run->len -= newfill->len;
+                    data_run_cur->len -= newfill->len;
                 }
 
-                /* get to the end of the run that we are trying
-                 * to insert
-                 */
-                endrun = run;
+                /* get to the end of the run that we are trying to add */
+                endrun = data_run_new;
                 while (endrun->next)
                     endrun = endrun->next;
 
                 /* if the filler is the same size as the
                  * new one, replace it 
                  */
-                if (runlen == data_run->len) {
-                    endrun->next = data_run->next;
-                    free(data_run);
+                if (runlen == data_run_cur->len) {
+                    endrun->next = data_run_cur->next;
+
+                    // update the pointer to the end of the list
+                    if (endrun->next == NULL)
+                        fs_data->run_end = endrun;
+
+                    free(data_run_cur);
                 }
                 /* else adjust the last filler entry */
                 else {
-                    endrun->next = data_run;
-                    data_run->len -= runlen;
+                    endrun->next = data_run_cur;
+                    data_run_cur->len -= runlen;
                 }
 
                 return fs_data_head;
             }
         }
 
-        cur_vcn += data_run->len;
-        data_run_prev = data_run;
-        data_run = data_run->next;
+        data_run_prev = data_run_cur;
+        data_run_cur = data_run_cur->next;
     }
 
 
@@ -609,7 +652,7 @@ tsk_fs_data_put_run(TSK_FS_DATA * fs_data_head,
      * we got here because it did not fit in the current list or
      * because the current list is NULL
      *
-     * At this point cur_vcn is the end of the existing list of
+     * At this point data_run_prev is the end of the existing list or
      * 0 if there is no list
      */
 
@@ -618,13 +661,14 @@ tsk_fs_data_put_run(TSK_FS_DATA * fs_data_head,
      * what we are inserting, but we never found the filler
      * for where we were to insert
      */
-    if (cur_vcn > start_vcn) {
+    if ((data_run_prev)
+        && (data_run_prev->offset + data_run_prev->len >
+            data_run_new->offset)) {
 
         /* MAYBE this is because of a duplicate entry .. */
-        if ((data_run_prev) &&
-            (data_run_prev->addr == run->addr) &&
-            (data_run_prev->len == run->len)) {
-            tsk_fs_data_run_free(run);
+        if ((data_run_prev->addr == data_run_new->addr) &&
+            (data_run_prev->len == data_run_new->len)) {
+            tsk_fs_data_run_free(data_run_new);
             return fs_data_head;
         }
 
@@ -633,17 +677,20 @@ tsk_fs_data_put_run(TSK_FS_DATA * fs_data_head,
         snprintf(tsk_errstr, TSK_ERRSTR_L,
             "fs_data_run: error adding aditional run: %" PRIuDADDR
             ", Previous %" PRIuDADDR " -> %" PRIuDADDR "   Current %"
-            PRIuDADDR " -> %" PRIuDADDR "\n", start_vcn,
-            data_run_prev->addr, data_run_prev->len, run->addr, run->len);
+            PRIuDADDR " -> %" PRIuDADDR "\n", data_run_new->offset,
+            data_run_prev->addr, data_run_prev->len, data_run_new->addr,
+            data_run_new->len);
         return NULL;
     }
 
     /* we should add it right here */
-    else if (cur_vcn == start_vcn) {
+    else if (((data_run_prev)
+            && (data_run_prev->offset + data_run_prev->len ==
+                data_run_new->offset)) || (data_run_new->offset == 0)) {
         if (data_run_prev)
-            data_run_prev->next = run;
+            data_run_prev->next = data_run_new;
         else
-            fs_data->run = run;
+            fs_data->run = data_run_new;
     }
     /* we need to make a filler before it */
     else {
@@ -651,13 +698,17 @@ tsk_fs_data_put_run(TSK_FS_DATA * fs_data_head,
         if (tmprun == NULL)
             return NULL;
 
-        if (data_run_prev)
+        if (data_run_prev) {
             data_run_prev->next = tmprun;
-        else
+            tmprun->offset = data_run_prev->offset + data_run_prev->len;
+        }
+        else {
             fs_data->run = tmprun;
-        tmprun->len = start_vcn - cur_vcn;
+        }
+
+        tmprun->len = data_run_new->offset - tmprun->offset;
         tmprun->flags = TSK_FS_DATA_RUN_FLAG_FILLER;
-        tmprun->next = run;
+        tmprun->next = data_run_new;
     }
 
     /* Adjust the length of the TSK_FS_DATA structure to reflect the 
@@ -665,6 +716,11 @@ tsk_fs_data_put_run(TSK_FS_DATA * fs_data_head,
      */
     fs_data->allocsize += runlen;
 
+    // update the pointer to the end of the list
+    fs_data->run_end = data_run_new;
+    while (fs_data->run_end->next)
+        fs_data->run_end = fs_data->run_end->next;
+
     /* return head of fs_data list */
     return fs_data_head;
 }
