@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2007/04/05 16:01:57 $
+ * $Date: 2007/12/05 13:52:56 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
@@ -67,12 +67,17 @@ tsk_fs_read_block(TSK_FS_INFO * fs, TSK_DATA_BUF * buf, OFF_T len,
         return -1;
     }
 
-    if (addr > fs->last_block) {
+    if (addr > fs->last_block_act) {
         tsk_error_reset();
         tsk_errno = TSK_ERR_FS_READ;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "tsk_fs_read_block: Address is too large: %" PRIuDADDR ")",
-            addr);
+        if (addr <= fs->last_block)
+            snprintf(tsk_errstr, TSK_ERRSTR_L,
+                "tsk_fs_read_block: Address missing in partial image: %"
+                PRIuDADDR ")", addr);
+        else
+            snprintf(tsk_errstr, TSK_ERRSTR_L,
+                "tsk_fs_read_block: Address is too large for image: %"
+                PRIuDADDR ")", addr);
         return -1;
     }
 
@@ -111,12 +116,17 @@ tsk_fs_read_block_nobuf(TSK_FS_INFO * fs, char *buf, OFF_T len,
         return -1;
     }
 
-    if (addr > fs->last_block) {
+    if (addr > fs->last_block_act) {
         tsk_error_reset();
         tsk_errno = TSK_ERR_FS_READ;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "tsk_fs_read_block: Address is too large: %" PRIuDADDR ")",
-            addr);
+        if (addr <= fs->last_block)
+            snprintf(tsk_errstr, TSK_ERRSTR_L,
+                "tsk_fs_read_block: Address missing in partial image: %"
+                PRIuDADDR ")", addr);
+        else
+            snprintf(tsk_errstr, TSK_ERRSTR_L,
+                "tsk_fs_read_block: Address is too large for image: %"
+                PRIuDADDR ")", addr);
         return -1;
     }
 
@@ -232,7 +242,7 @@ fs_read_file_act_data(TSK_FS_INFO * fs, DADDR_T addr, char *buf,
     size_t blk_offset;
 
     /* Is this block too early in the stream? */
-    if (buf1->offset_left > size) {
+    if (buf1->offset_left >= size) {
         buf1->offset_left -= size;
         return TSK_WALK_CONT;
     }
@@ -270,9 +280,10 @@ fs_read_file_act_aonly(TSK_FS_INFO * fs, DADDR_T addr, char *buf,
     FS_READ_FILE *buf1 = (FS_READ_FILE *) ptr;
     size_t cp_size;
     size_t blk_offset;
+    SSIZE_T cnt;
 
     /* Is this block too early in the stream? */
-    if (buf1->offset_left > size) {
+    if (buf1->offset_left >= size) {
         buf1->offset_left -= size;
         return TSK_WALK_CONT;
     }
@@ -286,6 +297,19 @@ fs_read_file_act_aonly(TSK_FS_INFO * fs, DADDR_T addr, char *buf,
     else
         cp_size = size - blk_offset;
 
+    /* Before we go ahead with this, check if we are beyond a partial
+     * image and give the "partial image" error message.  The AONLY flag
+     * allows us to get addresses that are not in a corrupt image file. 
+     */
+    if (addr > fs->last_block_act) {
+        tsk_error_reset();
+        tsk_errno = TSK_ERR_FS_READ;
+        snprintf(tsk_errstr, TSK_ERRSTR_L,
+            "tsk_fs_read_file: Address is too large for partial image: %"
+            PRIuDADDR ")", addr);
+        return TSK_WALK_ERROR;
+    }
+
     /* If the block is sparse, then simply write zeros */
     if (flags & TSK_FS_BLOCK_FLAG_SPARSE) {
         memset(buf1->cur, 0, cp_size);
@@ -308,8 +332,12 @@ fs_read_file_act_aonly(TSK_FS_INFO * fs, DADDR_T addr, char *buf,
              * except for compressed NTFS -- which do not use this callback)
              */
             if (cache_offset + cp_size > FS_READ_FILE_CACHE_SZ) {
-                tsk_fs_read_random(fs, buf1->cache, FS_READ_FILE_CACHE_SZ,
-                    (addr * fs->block_size));
+                cnt =
+                    tsk_fs_read_random(fs, buf1->cache,
+                    FS_READ_FILE_CACHE_SZ, (addr * fs->block_size));
+                if (cnt < 0) {
+                    return TSK_WALK_ERROR;
+                }
                 buf1->cache_base = addr;
                 cache_offset = blk_offset;
             }
@@ -321,8 +349,12 @@ fs_read_file_act_aonly(TSK_FS_INFO * fs, DADDR_T addr, char *buf,
          * if the size of each callback is less than the cache size. */
         else if ((buf1->size_left > fs->block_size) &&
             (size < FS_READ_FILE_CACHE_SZ)) {
-            tsk_fs_read_random(fs, buf1->cache, FS_READ_FILE_CACHE_SZ,
+            cnt =
+                tsk_fs_read_random(fs, buf1->cache, FS_READ_FILE_CACHE_SZ,
                 (addr * fs->block_size));
+            if (cnt < 0) {
+                return TSK_WALK_ERROR;
+            }
 
             buf1->cache_inuse = 1;
             buf1->cache_base = addr;
@@ -331,8 +363,11 @@ fs_read_file_act_aonly(TSK_FS_INFO * fs, DADDR_T addr, char *buf,
         /* Fallback case where we simply read into the buffer and ignore
          * the cache */
         else {
-            tsk_fs_read_random(fs, buf1->cur, cp_size,
+            cnt = tsk_fs_read_random(fs, buf1->cur, cp_size,
                 (addr * fs->block_size) + blk_offset);
+            if (cnt < 0) {
+                return TSK_WALK_ERROR;
+            }
         }
     }
 
@@ -354,13 +389,13 @@ fs_read_file_act_aonly(TSK_FS_INFO * fs, DADDR_T addr, char *buf,
  *
  * @param fs The file system structure.
  * @param fsi The inode structure of the file to read.
- * @param type The type of attribute to load (ignored if TSK_FS_FILE_FLAG_NOID is given)
+ * @param type The type of attribute to load (Use 0 to use the default attribute type)
  * @param id The id of attribute to load (ignored if TSK_FS_FILE_FLAG_NOID is given)
  * @param offset The byte offset to start reading from.
  * @param size The number of bytes to read from the file.
  * @param buf The buffer to read the data into.
  * @param flagsBase The base set of flags to set.
- * @returns The number of bytes read or -1 on error.
+ * @returns The number of bytes read or -1 on error.  0 is returned on EOF.
  */
 static SSIZE_T
 fs_read_file_int(TSK_FS_INFO * fs, TSK_FS_INODE * fsi, uint32_t type,
@@ -369,14 +404,41 @@ fs_read_file_int(TSK_FS_INFO * fs, TSK_FS_INODE * fsi, uint32_t type,
     FS_READ_FILE lf;
     int flags = flagsBase;
 
+    if (offset > fsi->size) {
+        return 0;
+    }
+
+    if (fsi->flags & TSK_FS_INODE_FLAG_UNALLOC) {
+        flags |= TSK_FS_FILE_FLAG_RECOVER;
+    }
+
+    // special handling for FAT 
+    if ((fs->ftype & TSK_FS_INFO_TYPE_FS_MASK) ==
+        TSK_FS_INFO_TYPE_FAT_TYPE) {
+        SSIZE_T offset2;
+
+        lf.base = lf.cur = buf;
+        lf.size_to_copy = lf.size_left = size;
+        lf.cache_inuse = 0;
+
+        offset2 = offset & ~(fs->block_size - 1);
+
+        lf.offset_left = offset - offset2;
+        if (fatfs_file_walk_off(fs, fsi, type, id, offset2, flags,
+                fs_read_file_act_data, (void *) &lf)) {
+            strncat(tsk_errstr2, " - tsk_fs_read_file",
+                TSK_ERRSTR_L - strlen(tsk_errstr2));
+            return -1;
+        }
+
+        return lf.size_to_copy - lf.size_left;
+    }
+
     lf.base = lf.cur = buf;
     lf.size_to_copy = lf.size_left = size;
     lf.offset_left = offset;
     lf.cache_inuse = 0;
 
-    if (fsi->flags & TSK_FS_INODE_FLAG_UNALLOC) {
-        flags |= TSK_FS_FILE_FLAG_RECOVER;
-    }
 
     /* For compressed files, we must do a normal walk.  For non-compressed
      * files, we can simply do an AONLY walk and then read only the blocks 
@@ -407,11 +469,16 @@ fs_read_file_int(TSK_FS_INFO * fs, TSK_FS_INODE * fsi, uint32_t type,
                 return -1;
             }
 
-            if ((fsi->mode & TSK_FS_INODE_MODE_FMT) ==
-                TSK_FS_INODE_MODE_DIR)
-                type = NTFS_ATYPE_IDXROOT;
-            else
-                type = NTFS_ATYPE_DATA;
+
+            /* If they did not give a type, we need to determine
+             * what the default should be. */
+            if (type == 0) {
+                if ((fsi->mode & TSK_FS_INODE_MODE_FMT) ==
+                    TSK_FS_INODE_MODE_DIR)
+                    type = NTFS_ATYPE_IDXROOT;
+                else
+                    type = NTFS_ATYPE_DATA;
+            }
 
             if (flags & TSK_FS_FILE_FLAG_NOID)
                 fs_data = tsk_fs_data_lookup_noid(fsi->attr, type);
