@@ -2,7 +2,7 @@
 ** tsk_fs_open
 ** The Sleuth Kit 
 **
-** $Date: 2007/04/05 16:01:57 $
+** $Date: 2007/11/30 16:44:20 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
@@ -35,14 +35,15 @@
  */
 
 /**
- * Read data and process it as a file system.
+ * Tries to open data (at a given offset) as a file system. 
+ * Returns a structure that can be used for analysis and reporting. 
  *
  * @param img_info Disk image to analyze
  * @param offset Byte offset to start analyzing from
  * @param type String type of file system type 
- * (autodetects if NULL)
+ * (autodetects if set to NULL)
  *
- * @return NULL on error
+ * @return NULL on error 
  */
 
 TSK_FS_INFO *
@@ -133,7 +134,7 @@ tsk_fs_open(TSK_IMG_INFO * img_info, SSIZE_T offset,
         }
 
 
-#if 0
+#if TSK_USE_HFS
         if ((fs_info =
                 hfs_open(img_info, offset, TSK_FS_INFO_TYPE_HFS,
                     1)) != NULL) {
