@@ -5,7 +5,7 @@
 ** This header file is to be included if file system routines
 ** are used from the library.  
 **
-** $Date: 2007/04/05 16:01:57 $
+** $Date: 2007/11/05 19:08:31 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2003-2007 Brian Carrier.  All rights reserved
@@ -44,43 +44,43 @@ extern "C" {
      * sub-type of implementation.  
      */
     enum TSK_FS_INFO_TYPE_ENUM {
-        TSK_FS_INFO_TYPE_FS_MASK = 0xf0,
-        TSK_FS_INFO_TYPE_SUB_MASK = 0x0f,
+        TSK_FS_INFO_TYPE_FS_MASK = 0xf0,        ///< Mask for general file system type
+        TSK_FS_INFO_TYPE_SUB_MASK = 0x0f,       ///< Mask for file system sub-type (version)
 
-        TSK_FS_INFO_TYPE_UNSUPP = 0x00,
+        TSK_FS_INFO_TYPE_UNSUPP = 0x00, ///< Unsupported file system
 
-        TSK_FS_INFO_TYPE_FFS_TYPE = 0x10,
-        TSK_FS_INFO_TYPE_FFS_1 = 0x11,  /* UFS1 - FreeBSD, OpenBSD, BSDI ... */
-        TSK_FS_INFO_TYPE_FFS_1B = 0x12, /* Solaris (no type) */
-        TSK_FS_INFO_TYPE_FFS_2 = 0x13,  /* UFS2 - FreeBSD, NetBSD */
-        TSK_FS_INFO_TYPE_FFS_AUTO = 0x14,
+        TSK_FS_INFO_TYPE_FFS_TYPE = 0x10,       ///< FFS General type
+        TSK_FS_INFO_TYPE_FFS_1 = 0x11,  ///< UFS1 (FreeBSD, OpenBSD, BSDI ...)
+        TSK_FS_INFO_TYPE_FFS_1B = 0x12, ///< UFS1b (Solaris - has no type)
+        TSK_FS_INFO_TYPE_FFS_2 = 0x13,  ///< UFS2 - FreeBSD, NetBSD 
+        TSK_FS_INFO_TYPE_FFS_AUTO = 0x14,       ///< UFS Autodetect - passed to ffs_open to detect specific type
 
-        TSK_FS_INFO_TYPE_EXT_TYPE = 0x20,
-        TSK_FS_INFO_TYPE_EXT_2 = 0x21,
-        TSK_FS_INFO_TYPE_EXT_3 = 0x22,
-        TSK_FS_INFO_TYPE_EXT_AUTO = 0x23,
+        TSK_FS_INFO_TYPE_EXT_TYPE = 0x20,       ///< Ext2/3 General type
+        TSK_FS_INFO_TYPE_EXT_2 = 0x21,  ///< Ext2 file system
+        TSK_FS_INFO_TYPE_EXT_3 = 0x22,  ///< Ext3 file system
+        TSK_FS_INFO_TYPE_EXT_AUTO = 0x23,       ///< Ext2/3 Autodetect - passed to ext2fs_open to detect specific type
 
-        TSK_FS_INFO_TYPE_FAT_TYPE = 0x30,
-        TSK_FS_INFO_TYPE_FAT_12 = 0x31,
-        TSK_FS_INFO_TYPE_FAT_16 = 0x32,
-        TSK_FS_INFO_TYPE_FAT_32 = 0x33,
-        TSK_FS_INFO_TYPE_FAT_AUTO = 0x34,
+        TSK_FS_INFO_TYPE_FAT_TYPE = 0x30,       ///< FAT general type
+        TSK_FS_INFO_TYPE_FAT_12 = 0x31, ///< FAT12 file system
+        TSK_FS_INFO_TYPE_FAT_16 = 0x32, ///< FAT16 file system
+        TSK_FS_INFO_TYPE_FAT_32 = 0x33, ///< FAT32 file system
+        TSK_FS_INFO_TYPE_FAT_AUTO = 0x34,       ///< FAT Autodetect - passed to fatfs_open to detect specific type
 
-        TSK_FS_INFO_TYPE_NTFS_TYPE = 0x40,
-        TSK_FS_INFO_TYPE_NTFS = 0x40,
-        TSK_FS_INFO_TYPE_NTFS_AUTO = 0x40,
+        TSK_FS_INFO_TYPE_NTFS_TYPE = 0x40,      ///< NTFS general type
+        TSK_FS_INFO_TYPE_NTFS = 0x40,   ///< NTFS file system
+        TSK_FS_INFO_TYPE_NTFS_AUTO = 0x40,      ///< NTFS Autodetect - passed to ntfs_open to detect type
 
-        TSK_FS_INFO_TYPE_SWAP_TYPE = 0x50,
-        TSK_FS_INFO_TYPE_SWAP = 0x50,
+        TSK_FS_INFO_TYPE_SWAP_TYPE = 0x50,      ///< SWAP general type
+        TSK_FS_INFO_TYPE_SWAP = 0x50,   ///< SWAP file system
 
-        TSK_FS_INFO_TYPE_RAW_TYPE = 0x60,
-        TSK_FS_INFO_TYPE_RAW = 0x60,
+        TSK_FS_INFO_TYPE_RAW_TYPE = 0x60,       ///< RAW general type
+        TSK_FS_INFO_TYPE_RAW = 0x60,    ///< RAW file system
 
-        TSK_FS_INFO_TYPE_ISO9660_TYPE = 0x70,
-        TSK_FS_INFO_TYPE_ISO9660 = 0x70,
+        TSK_FS_INFO_TYPE_ISO9660_TYPE = 0x70,   ///< ISO9660 general type
+        TSK_FS_INFO_TYPE_ISO9660 = 0x70,        ///< ISO9660 file system
 
-        TSK_FS_INFO_TYPE_HFS_TYPE = 0x80,
-        TSK_FS_INFO_TYPE_HFS = 0x80,
+        TSK_FS_INFO_TYPE_HFS_TYPE = 0x80,       ///< HFS+ general type
+        TSK_FS_INFO_TYPE_HFS = 0x80,    ///< HFS file system
     };
     typedef enum TSK_FS_INFO_TYPE_ENUM TSK_FS_INFO_TYPE_ENUM;
 
@@ -177,6 +177,12 @@ extern "C" {
     };
     typedef enum TSK_FS_INODE_FLAG_ENUM TSK_FS_INODE_FLAG_ENUM;
 
+    enum TSK_FS_INODE_ATTR_FLAG_ENUM {
+        TSK_FS_INODE_ATTR_EMPTY,        ///< Attributes have not been studied
+        TSK_FS_INODE_ATTR_STUDIED,      ///< Attributes have been studied successfully
+        TSK_FS_INODE_ATTR_ERROR ///< Attributes have been studied and errors occured
+    };
+    typedef enum TSK_FS_INODE_ATTR_FLAG_ENUM TSK_FS_INODE_ATTR_FLAG_ENUM;
 
 
     /**
@@ -234,7 +240,11 @@ extern "C" {
         int indir_count;        ///< Number of blocks in indirect list
 
         uint32_t seq;           ///< Sequence number of file (NTFS Only) 
-        TSK_FS_DATA *attr;      ///< Attributes for file (NTFS Only) 
+
+        TSK_FS_DATA *attr;      ///< Run for content (or NTFS file attribute) 
+        TSK_FS_INODE_ATTR_FLAG_ENUM attr_state; ///< State of the attribute structure
+        uint8_t attr_flags;     ///< Identifies flags used to load attribute run (FAT Only right now -- RECOVERY or not RECOVERY)
+
         TSK_FS_INODE_NAME_LIST *name;   ///< Name of file stored in metadata (FAT and NTFS Only)
         char *link;             ///< Name of target file if this is a symbolic link
 
@@ -374,7 +384,8 @@ extern "C" {
         /* content */
         DADDR_T block_count;    ///< Number of blocks in fs
         DADDR_T first_block;    ///< Address of first block
-        DADDR_T last_block;     ///< Address of last block
+        DADDR_T last_block;     ///< Address of last block as reported by file system (could be larger than last_block if end of image does not exist)
+        DADDR_T last_block_act; ///< Address of last block -- adjusted so that it is equal to the last block in the image or volume (if image is not complete)
         unsigned int block_size;        ///< Size of each block (in bytes)
         unsigned int dev_bsize; ///< Size of device block (typically always 512)
 
@@ -392,27 +403,83 @@ extern "C" {
 					 * Used to find orphans
 					 */
 
-        /** Walk a set of blocks and pass each to the callback */
-         uint8_t(*block_walk) (TSK_FS_INFO *, DADDR_T, DADDR_T,
-            TSK_FS_BLOCK_FLAG_ENUM, TSK_FS_BLOCK_WALK_CB, void *);
+        /** Pointer to file system-specific function that passes a range of blocks to a callback function. 
+         * @param fs File system to analyze
+         * @param start Starting block in range to walk from
+         * @param end Ending block in range to walk to
+         * @param flags Flags for walking
+         * @param cb Callback function
+         * @param ptr Pointer to data that is passed to callback
+         * @returns 1 on error and 0 on success
+         */
+         uint8_t(*block_walk) (TSK_FS_INFO * fs, DADDR_T start,
+            DADDR_T end, TSK_FS_BLOCK_FLAG_ENUM flags,
+            TSK_FS_BLOCK_WALK_CB cb, void *ptr);
 
-        /** Walk a set of inodes and pass each to the callback */
-         uint8_t(*inode_walk) (TSK_FS_INFO *, INUM_T, INUM_T,
-            TSK_FS_INODE_FLAG_ENUM, TSK_FS_INODE_WALK_CB, void *);
+        /** Pointer to file system-specific function that passes a range of inodes to a callback function. 
+         * @param fs File system to analyze
+         * @param start Starting inode in range to walk from
+         * @param end Ending inode in range to walk to
+         * @param flags Flags for walking
+         * @param cb Callback function
+         * @param ptr Pointer to data that is passed to callback
+         * @returns 1 on error and 0 on success
+         */
+         uint8_t(*inode_walk) (TSK_FS_INFO * fs, INUM_T start, INUM_T end,
+            TSK_FS_INODE_FLAG_ENUM flags, TSK_FS_INODE_WALK_CB cb,
+            void *ptr);
 
-        /** Lookup an inode and return it */
-        TSK_FS_INODE *(*inode_lookup) (TSK_FS_INFO *, INUM_T);
+        /** Pointer to file system-specific function that looks up a file and returns the info in a generic format.
+         * @param fs File system to analyze
+         * @param addr Metadata address of file to lookup
+         * @returns NULL on error
+         */
+        TSK_FS_INODE *(*inode_lookup) (TSK_FS_INFO * fs, INUM_T addr);
 
-        /** Print file details to a handle */
-         uint8_t(*istat) (TSK_FS_INFO *, FILE *, INUM_T, DADDR_T, int32_t);
+        /**
+         * Pointer to file system specific function that prints details on a specific file to a file handle. 
+         *
+         * @param fs File system file is located in
+         * @param hFile File handle to print text to
+         * @param inum Address of file in file system
+         * @param numblock The number of blocks in file to force print (can go beyond file size)
+         * @param sec_skew Clock skew in seconds to also print times in
+         * 
+         * @returns 1 on error and 0 on success
+         */
+         uint8_t(*istat) (TSK_FS_INFO * fs, FILE * hFile, INUM_T inum,
+            DADDR_T numblock, int32_t sec_skew);
 
-        /** Walk the contents of a file and pass each block to the callback */
-         uint8_t(*file_walk) (TSK_FS_INFO *, TSK_FS_INODE *, uint32_t,
-            uint16_t, TSK_FS_FILE_FLAG_ENUM, TSK_FS_FILE_WALK_CB, void *);
+         /**
+         * Pointer to a file system-specific function that calls a callback function with the contents of each block in a file. 
+         *
+         * @param fs File system file is located in
+         * @param fs_inode File to read and analyze
+         * @param type Attribute type to read and analyze (does not apply to all file systems)
+         * @param id Attribute id to read and analyze (does not apply to all file systems)
+         * @param flags Flags to use while reading
+         * @param cb Callback function that is called for each block
+         * @param ptr Pointer to data that is passed to the callback
+         * @returns 1 on error and 0 on success
+         */
+         uint8_t(*file_walk) (TSK_FS_INFO * fs, TSK_FS_INODE * fs_inode,
+            uint32_t type, uint16_t id, TSK_FS_FILE_FLAG_ENUM flags,
+            TSK_FS_FILE_WALK_CB cb, void *ptr);
 
-        /** Walk the files in a directory and pass each file name to the callback */
-         uint8_t(*dent_walk) (TSK_FS_INFO *, INUM_T, TSK_FS_DENT_FLAG_ENUM,
-            TSK_FS_DENT_TYPE_WALK_CB, void *);
+
+        /**
+         * Pointer to file system-specific function that processes the contents of a directory and passes each file name to a callback function.
+         *
+         * @param fs File system to analyze
+         * @param inode Metadata address of directory to analyze
+         * @param flags Flags used during analysis
+         * @param action Callback function that is called for each file name
+         * @param ptr Pointer to data that is passed to callback
+         * @returns 1 on error and 0 on success
+         */
+         uint8_t(*dent_walk) (TSK_FS_INFO * fs, INUM_T inode,
+            TSK_FS_DENT_FLAG_ENUM flags, TSK_FS_DENT_TYPE_WALK_CB action,
+            void *ptr);
 
         /** Open the journal */
          uint8_t(*jopen) (TSK_FS_INFO *, INUM_T);
@@ -425,15 +492,24 @@ extern "C" {
          uint8_t(*jentry_walk) (TSK_FS_INFO *, int, TSK_FS_JENTRY_WALK_CB,
             void *);
 
-
-        /** Print the file system details to FILE handle */
-         uint8_t(*fsstat) (TSK_FS_INFO *, FILE *);
+         /**
+         * Pointer to file system-specific function that prints file system details to a file handle.
+         *
+         * @param fs File system to print details on
+         * @param hFile File handle to print text to
+         * 
+         * @returns 1 on error and 0 on success
+         */
+         uint8_t(*fsstat) (TSK_FS_INFO * fs, FILE * hFile);
 
         /** Check the integrity / sanity of the file system (not implemented) */
          uint8_t(*fscheck) (TSK_FS_INFO *, FILE *);
 
-        /** Close the file system and free the allocated memory */
-        void (*close) (TSK_FS_INFO *);
+        /** 
+         * Pointer to file system-specific function that closes the file system and frees the allocated memory 
+         * @param fs File system to close
+         */
+        void (*close) (TSK_FS_INFO * fs);
     };
 
 
@@ -470,6 +546,7 @@ extern "C" {
      */
     struct TSK_FS_DATA_RUN {
         TSK_FS_DATA_RUN *next;  ///< Pointer to the next run in the attribute
+        DADDR_T offset;         ///< Logical block offset of this run in the "file"
         DADDR_T addr;           ///< Starting block address of run
         DADDR_T len;            ///< Number of blocks in run
         TSK_FS_DATA_RUN_FLAG_ENUM flags;        ///< Flags for run
@@ -486,6 +563,7 @@ extern "C" {
         TSK_FS_DATA_ENC = 0x10, ///< Attribute data is encrypted
         TSK_FS_DATA_COMP = 0x20,        ///< Attribute data is compressed
         TSK_FS_DATA_SPARSE = 0x40,      ///< Attribute data is sparse
+        TSK_FS_DATA_RECOVERY = 0x80,    ///< Attribute layout was determined in recovery mode
     };
     typedef enum TSK_FS_DATA_FLAG_ENUM TSK_FS_DATA_FLAG_ENUM;
 
@@ -509,6 +587,7 @@ extern "C" {
 
         /* Run-List data (non-resident) */
         TSK_FS_DATA_RUN *run;   ///< Linked list of runs for non-resident attributes
+        TSK_FS_DATA_RUN *run_end;       ///< Pointer to final run in the list
         OFF_T allocsize;        ///< Number of bytes that are allocated in all clusters of non-resident run (will be larger than size)
         uint32_t compsize;      ///< Size of compression units (needed only if file is compressed)
 
@@ -528,7 +607,7 @@ extern "C" {
     extern TSK_FS_DATA *tsk_fs_data_put_str(TSK_FS_DATA *, char *,
         uint32_t, uint16_t, void *, unsigned int);
 
-    extern TSK_FS_DATA *tsk_fs_data_put_run(TSK_FS_DATA *, DADDR_T, OFF_T,
+    extern TSK_FS_DATA *tsk_fs_data_put_run(TSK_FS_DATA *, OFF_T,
         TSK_FS_DATA_RUN *, char *, uint32_t, uint16_t, OFF_T,
         TSK_FS_DATA_FLAG_ENUM, uint32_t);
 
@@ -668,6 +747,13 @@ extern "C" {
 
 
 
+    extern uint8_t
+        fatfs_file_walk_off(TSK_FS_INFO * fs, TSK_FS_INODE * fs_inode,
+        uint32_t type, uint16_t id, OFF_T a_st_off,
+        TSK_FS_FILE_FLAG_ENUM flags, TSK_FS_FILE_WALK_CB action,
+        void *ptr);
+
+
 #ifdef __cplusplus
 }
 #endif
