@@ -2,7 +2,7 @@
 ** fs_types
 ** The Sleuth Kit 
 **
-** $Date: 2007/04/04 20:06:57 $
+** $Date: 2007/11/30 16:44:20 $
 **
 ** Identify the type of file system being used
 **
@@ -18,17 +18,27 @@
 **
 */
 
+/**
+ * \file fs_types.c
+ * Contains functions that deal with parsing and printing file system type strings.
+ */
+
+
 #include "fs_tools_i.h"
 
 /* Based on tsk_fs_open.c in TCT-1.07 */
 
+/**
+ * \internal
+ */
 typedef struct {
     char *name;
     TSK_FS_INFO_TYPE_ENUM code;
     char *comment;
 } FS_TYPES;
 
-/* The table used to parse input strings - supports
+/** \internal
+ * The table used to parse input strings - supports
  * legacy strings - in order of expected usage
  */
 FS_TYPES fs_open_table[] = {
@@ -36,7 +46,9 @@ FS_TYPES fs_open_table[] = {
     {"fat", TSK_FS_INFO_TYPE_FAT_AUTO, "auto-detect FAT"},
     {"ext", TSK_FS_INFO_TYPE_EXT_AUTO, "Ext2/Ext3"},
     {"iso9660", TSK_FS_INFO_TYPE_ISO9660, "ISO9660 CD"},
-//    {"hfs", HFS, "HFS+"},
+#if TSK_USE_HFS
+    {"hfs", TSK_FS_INFO_TYPE_HFS, "HFS+"},
+#endif
     {"ufs", TSK_FS_INFO_TYPE_FFS_AUTO, "UFS 1 & 2"},
     {"raw", TSK_FS_INFO_TYPE_RAW, "Raw Data"},
     {"swap", TSK_FS_INFO_TYPE_SWAP, "Swap Space"},
@@ -54,13 +66,18 @@ FS_TYPES fs_open_table[] = {
     {0},
 };
 
-/* Used to print the name given the code */
+/**
+ * \internal
+ * Used to print the name given the code 
+ */
 FS_TYPES fs_test_table[] = {
     {"ntfs", TSK_FS_INFO_TYPE_NTFS_AUTO, ""},
     {"fat", TSK_FS_INFO_TYPE_FAT_AUTO, ""},
     {"ext", TSK_FS_INFO_TYPE_EXT_AUTO, ""},
     {"ufs", TSK_FS_INFO_TYPE_FFS_AUTO, ""},
-//    {"hfs", HFS, ""},
+#if TSK_USE_HFS
+    {"hfs", TSK_FS_INFO_TYPE_HFS, ""},
+#endif
     {"iso9660", TSK_FS_INFO_TYPE_ISO9660, ""},
     {"raw", TSK_FS_INFO_TYPE_RAW, ""},
     {"swap", TSK_FS_INFO_TYPE_SWAP, ""},
@@ -75,11 +92,16 @@ FS_TYPES fs_test_table[] = {
     {0},
 };
 
+/**
+ * \internal
+ */
 FS_TYPES fs_usage_table[] = {
     {"ext", 0, "Ext2/Ext3"},
     {"fat", 0, "TSK_FS_INFO_TYPE_FAT_12/16/32"},
     {"ntfs", 0, "NTFS"},
-    //{"hfs", 0, "HFS+"},
+#if TSK_USE_HFS
+    {"hfs", 0, "HFS+"},
+#endif
     {"iso9660", 0, "ISO9660 CD"},
     {"ufs", 0, "UFS 1 & 2"},
     {"raw", 0, "Raw Data"},
@@ -88,6 +110,11 @@ FS_TYPES fs_usage_table[] = {
 };
 
 
+/**
+ * Parse a string (from the command line) and return the file system type id
+ * @param str File system type string
+ * @returns File system type or UNSUPP type on error
+ */
 TSK_FS_INFO_TYPE_ENUM
 tsk_fs_parse_type(const TSK_TCHAR * str)
 {
@@ -110,7 +137,10 @@ tsk_fs_parse_type(const TSK_TCHAR * str)
 }
 
 
-/* Used by the usage functions to display supported types */
+/**
+ * Print the supported file system types to a file handle
+ * @param hFile File handle to print to
+ */
 void
 tsk_fs_print_types(FILE * hFile)
 {
@@ -120,6 +150,11 @@ tsk_fs_print_types(FILE * hFile)
         tsk_fprintf(hFile, "\t%s (%s)\n", sp->name, sp->comment);
 }
 
+/**
+ * Return the string name of a file system type id.
+ * @param ftype File system type id
+ * @returns Name or NULL on error
+ */
 char *
 tsk_fs_get_type(TSK_FS_INFO_TYPE_ENUM ftype)
 {
