@@ -137,10 +137,10 @@ hfs_cat_find_node_offset(HFS_INFO * hfs, int nodenum)
 
     bytes =
         tsk_getu32(fs->endian,
-        sb->cat_file.extents[i].blk_cnt) * fs->block_size;
+        sb->cat_file.extents[i].blk_cnt) * (OFF_T) fs->block_size;
     r_offs =
         tsk_getu32(fs->endian,
-        sb->cat_file.extents[i].start_blk) * fs->block_size;
+        sb->cat_file.extents[i].start_blk) * (OFF_T) fs->block_size;
     f_offs = 0;
 
     nodesize = tsk_getu16(fs->endian, hfs->hdr->size);
@@ -167,11 +167,14 @@ hfs_cat_find_node_offset(HFS_INFO * hfs, int nodenum)
 
             r_offs =
                 tsk_getu32(fs->endian,
-                sb->cat_file.extents[i].start_blk) * fs->block_size;
+                sb->cat_file.extents[i].start_blk) *
+                (OFF_T) fs->block_size;
             f_offs += bytes;
+            // @@@ This was getu64, but Rob Joyce reported that it should
+            // be getu32.  Verify.
             bytes =
-                tsk_getu64(fs->endian,
-                sb->cat_file.extents[i].blk_cnt) * fs->block_size;
+                tsk_getu32(fs->endian,
+                sb->cat_file.extents[i].blk_cnt) * (OFF_T) fs->block_size;
 
         }
     }
@@ -925,6 +928,11 @@ hfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
     tsk_fprintf(hFile, "Block Range: %" PRIuDADDR " - %" PRIuDADDR "\n",
         fs->first_block, fs->last_block);
 
+    if (fs->last_block != fs->last_block_act)
+        tsk_fprintf(hFile,
+            "Total Range in Image: %" PRIuDADDR " - %" PRIuDADDR "\n",
+            fs->first_block, fs->last_block_act);
+
     tsk_fprintf(hFile, "Allocation Block Size: %u\n", fs->block_size);
 
     tsk_fprintf(hFile, "Free Blocks: %" PRIu32 "\n",
@@ -1075,7 +1083,7 @@ hfs_open(TSK_IMG_INFO * img_info, SSIZE_T offset,
 
     cnt = tsk_fs_read_random(fs, (char *) hfs->fs, len, (OFF_T) HFS_SBOFF);
     if (cnt != len) {
-        if (cnt != -1) {
+        if (cnt >= 0) {
             tsk_errno = TSK_ERR_FS_READ;
             tsk_errstr[0] = '\0';
         }
@@ -1101,11 +1109,16 @@ hfs_open(TSK_IMG_INFO * img_info, SSIZE_T offset,
 
     fs->block_count = tsk_getu32(fs->endian, hfs->fs->blk_cnt);
     fs->first_block = 0;
-    fs->last_block = fs->block_count - 1;
+    fs->last_block = fs->last_block_act = fs->block_count - 1;
 
     fs->dev_bsize = fs->block_size =
         tsk_getu32(fs->endian, hfs->fs->blk_sz);
 
+    // determine the last block we have in this image
+    if ((img_info->size - offset) / fs->block_size < fs->last_block)
+        fs->last_block_act =
+            (img_info->size - offset) / fs->block_size - 1;
+
     /*
      * Other initialization: caches, callbacks.
      */
