@@ -2,7 +2,7 @@
 ** icat 
 ** The Sleuth Kit 
 **
-** $Date: 2007/04/04 18:18:53 $
+** $Date: 2007/08/16 16:14:26 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
@@ -142,6 +142,12 @@ MAIN(int argc, TSK_TCHAR ** argv)
         tsk_error_print(stderr);
         exit(1);
     }
+    if (imgoff >= img->size) {
+        tsk_fprintf(stderr,
+            "Sector offset supplied is larger than disk image (maximum: %"
+            PRIu64 ")\n", img->size / 512);
+        exit(1);
+    }
     if ((fs = tsk_fs_open(img, imgoff, fstype)) == NULL) {
         tsk_error_print(stderr);
         if (tsk_errno == TSK_ERR_FS_UNSUPTYPE)
