@@ -2,7 +2,7 @@
 ** ifind (inode find)
 ** The Sleuth Kit
 **
-** $Date: 2007/04/05 16:01:58 $
+** $Date: 2007/08/16 16:14:26 $
 **
 ** Given an image  and block number, identify which inode it is used by
 ** 
@@ -202,6 +202,12 @@ MAIN(int argc, TSK_TCHAR ** argv)
             free(path);
         exit(1);
     }
+    if (imgoff >= img->size) {
+        tsk_fprintf(stderr,
+            "Sector offset supplied is larger than disk image (maximum: %"
+            PRIu64 ")\n", img->size / 512);
+        exit(1);
+    }
 
     if ((fs = tsk_fs_open(img, imgoff, fstype)) == NULL) {
         tsk_error_print(stderr);
