@@ -2,7 +2,7 @@
 ** ifind (inode find)
 ** The Sleuth Kit
 **
-** $Date: 2007/06/12 19:56:19 $
+** $Date: 2007/08/16 16:14:26 $
 **
 ** Given an image  and block number, identify which inode it is used by
 ** 
@@ -24,16 +24,16 @@
 #include "fs_tools_i.h"
 
 
-static uint8_t localflags;
-static uint8_t found;
+static uint8_t s_localflags;
+static uint8_t s_found;
 
 
 /*******************************************************************************
  * Find an unallocated NTFS MFT entry based on its parent directory
  */
 
-static TSK_FS_DENT *fs_dent = NULL;
-static INUM_T parinode = 0;
+static TSK_FS_DENT *s_fs_dent = NULL;
+static INUM_T s_parinode = 0;
 
 /* dent call back for finding unallocated files based on parent directory
  */
@@ -45,21 +45,21 @@ ifind_par_act(TSK_FS_INFO * fs, TSK_FS_INODE * fs_inode, void *ptr)
     /* go through each file name structure */
     fs_name = fs_inode->name;
     while (fs_name) {
-        if (fs_name->par_inode == parinode) {
+        if (fs_name->par_inode == s_parinode) {
             /* Fill in the basics of the fs_dent entry */
-            fs_dent->fsi = fs_inode;
-            fs_dent->inode = fs_inode->addr;
-            fs_dent->flags = TSK_FS_DENT_FLAG_UNALLOC;
-            strncpy(fs_dent->name, fs_name->name, fs_dent->name_max);
-            if (localflags & TSK_FS_IFIND_PAR_LONG) {
-                tsk_fs_dent_print_long(stdout, fs_dent, fs, NULL);
+            s_fs_dent->fsi = fs_inode;
+            s_fs_dent->inode = fs_inode->addr;
+            s_fs_dent->flags = TSK_FS_DENT_FLAG_UNALLOC;
+            strncpy(s_fs_dent->name, fs_name->name, s_fs_dent->name_max);
+            if (s_localflags & TSK_FS_IFIND_PAR_LONG) {
+                tsk_fs_dent_print_long(stdout, s_fs_dent, fs, NULL);
             }
             else {
-                tsk_fs_dent_print(stdout, fs_dent, fs, NULL);
+                tsk_fs_dent_print(stdout, s_fs_dent, fs, NULL);
                 tsk_printf("\n");
             }
-            fs_dent->fsi = NULL;
-            found = 1;
+            s_fs_dent->fsi = NULL;
+            s_found = 1;
         }
         fs_name = fs_name->next;
     }
@@ -73,21 +73,21 @@ ifind_par_act(TSK_FS_INFO * fs, TSK_FS_INODE * fs_inode, void *ptr)
 uint8_t
 tsk_fs_ifind_par(TSK_FS_INFO * fs, uint8_t lclflags, INUM_T par)
 {
-    found = 0;
-    localflags = lclflags;
-    parinode = par;
-    fs_dent = tsk_fs_dent_alloc(256, 0);
-    if (fs_dent == NULL)
+    s_found = 0;
+    s_localflags = lclflags;
+    s_parinode = par;
+    s_fs_dent = tsk_fs_dent_alloc(256, 0);
+    if (s_fs_dent == NULL)
         return 1;
 
     /* Walk unallocated MFT entries */
     if (fs->inode_walk(fs, fs->first_inum, fs->last_inum,
             TSK_FS_INODE_FLAG_UNALLOC, ifind_par_act, NULL)) {
-        tsk_fs_dent_free(fs_dent);
+        tsk_fs_dent_free(s_fs_dent);
         return 1;
     }
 
-    tsk_fs_dent_free(fs_dent);
+    tsk_fs_dent_free(s_fs_dent);
     return 0;
 }
 
@@ -100,8 +100,8 @@ tsk_fs_ifind_par(TSK_FS_INFO * fs, uint8_t lclflags, INUM_T par)
 #define IFIND_PATH_DATA_ID	0x00886644
 typedef struct {
     int id;
-    char *cur_dir;
-    char *cur_attr;
+    char *cur_dir;              ///< The "current" directory or file we are looking for
+    char *cur_attr;             ///< The "current" attribute of the dir we are looking for
     char *strtok_last;
     uint8_t found;
     uint8_t badpath;
@@ -115,7 +115,7 @@ typedef struct {
  * the needed directory is found
  */
 static uint8_t
-ifind_path_act(TSK_FS_INFO * fs, TSK_FS_DENT * fs_dent, void *ptr)
+ifind_path_act(TSK_FS_INFO * fs, TSK_FS_DENT * a_fs_dent, void *ptr)
 {
     IFIND_PATH_DATA *ipd = (IFIND_PATH_DATA *) ptr;
     char *pname;
@@ -135,7 +135,7 @@ ifind_path_act(TSK_FS_INFO * fs, TSK_FS_DENT * fs_dent, void *ptr)
         tsk_error_reset();
         tsk_errno = TSK_ERR_FS_ARG;
         snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "ifind: cur_dir is null: Please run with '-v' and send output to developers\n");
+            "ifind: cur_dir is null: Please run with '-v' and send output to carrier <at> sleuthkit <dot> org\n");
         return TSK_WALK_ERROR;
     }
 
@@ -149,7 +149,7 @@ ifind_path_act(TSK_FS_INFO * fs, TSK_FS_DENT * fs_dent, void *ptr)
             TSK_FS_INFO_TYPE_EXT_TYPE)
         || ((fs->ftype & TSK_FS_INFO_TYPE_FS_MASK) ==
             TSK_FS_INFO_TYPE_FFS_TYPE)) {
-        if (strcmp(fs_dent->name, ipd->cur_dir) != 0) {
+        if (strcmp(a_fs_dent->name, ipd->cur_dir) != 0) {
             return TSK_WALK_CONT;
         }
     }
@@ -157,7 +157,7 @@ ifind_path_act(TSK_FS_INFO * fs, TSK_FS_DENT * fs_dent, void *ptr)
     /* NTFS gets a case insensitive comparison */
     else if ((fs->ftype & TSK_FS_INFO_TYPE_FS_MASK) ==
         TSK_FS_INFO_TYPE_NTFS_TYPE) {
-        if (strcasecmp(fs_dent->name, ipd->cur_dir) != 0) {
+        if (strcasecmp(a_fs_dent->name, ipd->cur_dir) != 0) {
             return TSK_WALK_CONT;
         }
 
@@ -165,10 +165,10 @@ ifind_path_act(TSK_FS_INFO * fs, TSK_FS_DENT * fs_dent, void *ptr)
         if (ipd->cur_attr != NULL) {
             int fail = 1;
 
-            if (fs_dent->fsi) {
+            if (a_fs_dent->fsi) {
                 TSK_FS_DATA *fs_data;
 
-                for (fs_data = fs_dent->fsi->attr;
+                for (fs_data = a_fs_dent->fsi->attr;
                     fs_data != NULL; fs_data = fs_data->next) {
 
                     if ((fs_data->flags & TSK_FS_DATA_INUSE) == 0)
@@ -183,7 +183,7 @@ ifind_path_act(TSK_FS_INFO * fs, TSK_FS_DENT * fs_dent, void *ptr)
             if (fail) {
                 tsk_printf("Attribute name (%s) not found in %s: %"
                     PRIuINUM "\n", ipd->cur_attr, ipd->cur_dir,
-                    fs_dent->inode);
+                    a_fs_dent->inode);
 
                 return TSK_WALK_STOP;
             }
@@ -194,37 +194,57 @@ ifind_path_act(TSK_FS_INFO * fs, TSK_FS_DENT * fs_dent, void *ptr)
      */
     else if ((fs->ftype & TSK_FS_INFO_TYPE_FS_MASK) ==
         TSK_FS_INFO_TYPE_FAT_TYPE) {
-        if (strcasecmp(fs_dent->name, ipd->cur_dir) != 0) {
-            if (strcasecmp(fs_dent->shrt_name, ipd->cur_dir) != 0) {
+        if (strcasecmp(a_fs_dent->name, ipd->cur_dir) != 0) {
+            if (strcasecmp(a_fs_dent->shrt_name, ipd->cur_dir) != 0) {
                 return TSK_WALK_CONT;
             }
         }
     }
 
-    /* Get the next directory or file name in the path */
-    pname = ipd->cur_dir;   // save a copy of the current name pointer
+    /* If we made it here, then this file or directory was what
+     * we were looking for.  So, we move on to the next. 
+     */
+    pname = ipd->cur_dir;       // save a copy of the current name pointer
     ipd->cur_dir = (char *) strtok_r(NULL, "/", &(ipd->strtok_last));
     ipd->cur_attr = NULL;
 
     if (tsk_verbose)
         tsk_fprintf(stderr, "Found it (%s), now looking for %s\n",
-            fs_dent->name, ipd->cur_dir);
+            a_fs_dent->name, ipd->cur_dir);
 
-    /* That was the last name in the path -- we found the file */
+    /* That was the last name in the path -- we found the file! */
     if (ipd->cur_dir == NULL) {
-        //tsk_printf("%" PRIuINUM "\n", fs_dent->inode);
         ipd->found = 1;
-        ipd->addr = fs_dent->inode;
+        ipd->addr = a_fs_dent->inode;
 
-        // if our only hit is an unallocated entry 
-        // then keep on looking -- this commonly happens with NTFS
-        if (fs_dent->flags & TSK_FS_DENT_FLAG_UNALLOC) {
+        /* if our only hit is an unallocated entry 
+         * then keep on looking -- this commonly happens with NTFS */
+        if (a_fs_dent->flags & TSK_FS_DENT_FLAG_UNALLOC) {
             ipd->cur_dir = pname;
             return TSK_WALK_CONT;
         }
-        else {
-            return TSK_WALK_STOP;
-        }
+
+        return TSK_WALK_STOP;
+    }
+    /* There are more dirs / files to find */
+    /* Make sure this name has metadata / content */
+    else if (a_fs_dent->fsi == NULL) {
+        if (tsk_verbose)
+            tsk_fprintf(stderr, "Name does not point to an inode (%s)\n",
+                a_fs_dent->name);
+        return TSK_WALK_STOP;
+    }
+
+    /* Make sure this name is for a directory */
+    else if ((a_fs_dent->fsi->mode & TSK_FS_INODE_MODE_FMT) !=
+        TSK_FS_INODE_MODE_DIR) {
+        if (tsk_verbose)
+            tsk_fprintf(stderr,
+                "Name is not for a directory (%s) (mode: %x)\n",
+                a_fs_dent->name,
+                a_fs_dent->fsi->mode & TSK_FS_INODE_MODE_FMT);
+        ipd->badpath = 1;
+        return TSK_WALK_STOP;
     }
 
     /* if it is an NTFS image with an ADS in the name, then
@@ -237,39 +257,37 @@ ifind_path_act(TSK_FS_INFO * fs, TSK_FS_DENT * fs_dent, void *ptr)
         ipd->cur_attr++;
     }
 
-    /* it is a directory so we can recurse */
-    if ((fs_dent->fsi->mode & TSK_FS_INODE_MODE_FMT) ==
-        TSK_FS_INODE_MODE_DIR) {
-
-        if (fs->dent_walk(fs, fs_dent->inode,
-                TSK_FS_DENT_FLAG_ALLOC | TSK_FS_DENT_FLAG_UNALLOC,
-                ifind_path_act, (void *) ipd)) {
-            return TSK_WALK_ERROR;
-        }
-    }
-
-    /* The name was correct, but it was not a directory */
-    else {
-        ipd->badpath = 1;
+    /* Recurse on the directory */
+    if (fs->dent_walk(fs, a_fs_dent->inode,
+            TSK_FS_DENT_FLAG_ALLOC | TSK_FS_DENT_FLAG_UNALLOC,
+            ifind_path_act, (void *) ipd)) {
+        return TSK_WALK_ERROR;
     }
 
     return TSK_WALK_STOP;
 }
 
 
-/* Return -1 for error, 0 if found, and 1 if not found */
+/**
+ * Find the meta data address for a given file name
+ *
+ * @param fs FS to analyze
+ * @param lclflags Flags for search
+ * @param tpath Path of file to search for
+ * @param [out] result Meta data address of file
+ * @returns -1 on error, 0 if found, and 1 if not found
+ */
 int8_t
 tsk_fs_ifind_path(TSK_FS_INFO * fs, uint8_t lclflags, TSK_TCHAR * tpath,
     INUM_T * result)
 {
-    char *cpath;
+    char *cpath;                // utf-8 version of path to find
     IFIND_PATH_DATA ipd;
 
-
-    localflags = lclflags;
-
+    s_localflags = lclflags;
 
 #ifdef TSK_WIN32
+    // Convert the UTF-16 path to UTF-8
     {
         size_t clen;
         UTF8 *ptr8;
@@ -277,8 +295,7 @@ tsk_fs_ifind_path(TSK_FS_INFO * fs, uint8_t lclflags, TSK_TCHAR * tpath,
         int retval;
 
         clen = TSTRLEN(tpath) * 4;
-        cpath = (char *) tsk_malloc(clen);
-        if (cpath == NULL) {
+        if ((cpath = (char *) tsk_malloc(clen)) == NULL) {
             return -1;
         }
         ptr8 = (UTF8 *) cpath;
@@ -299,7 +316,14 @@ tsk_fs_ifind_path(TSK_FS_INFO * fs, uint8_t lclflags, TSK_TCHAR * tpath,
         }
     }
 #else
-    cpath = tpath;
+    {
+        // copy to a buffer that we can modify
+        size_t clen = strlen(tpath) + 1;
+        if ((cpath = (char *) tsk_malloc(clen)) == NULL) {
+            return -1;
+        }
+        strncpy(cpath, tpath, clen);
+    }
 #endif
 
     ipd.id = IFIND_PATH_DATA_ID;
@@ -310,14 +334,12 @@ tsk_fs_ifind_path(TSK_FS_INFO * fs, uint8_t lclflags, TSK_TCHAR * tpath,
 
     /* If there is no token, then only a '/' was given */
     if (!(ipd.cur_dir)) {
-#ifdef TSK_WIN32
         free(cpath);
-#endif
         *result = fs->root_inum;
         return 0;
     }
 
-    /* If this is NTFS, ensure that we take out the attribute */
+    /* If this is NTFS, seperate out the attribute of the current directory */
     if (((fs->ftype & TSK_FS_INFO_TYPE_FS_MASK) ==
             TSK_FS_INFO_TYPE_NTFS_TYPE)
         && ((ipd.cur_attr = strchr(ipd.cur_dir, ':')) != NULL)) {
@@ -334,9 +356,7 @@ tsk_fs_ifind_path(TSK_FS_INFO * fs, uint8_t lclflags, TSK_TCHAR * tpath,
         /* If we found files before the error was encountered, then 
          * ignore it */
         if (ipd.found == 0) {
-#ifdef TSK_WIN32
             free(cpath);
-#endif
             return -1;
         }
         else {
@@ -344,21 +364,18 @@ tsk_fs_ifind_path(TSK_FS_INFO * fs, uint8_t lclflags, TSK_TCHAR * tpath,
         }
     }
 
-
-#ifdef TSK_WIN32
     free(cpath);
-#endif
 
     if (1 == ipd.badpath) {
         if (tsk_verbose)
             tsk_fprintf(stderr, "Invalid path (%s is a file)\n",
-                fs_dent->name);
+                ipd.cur_dir);
         *result = 0;
         return 1;
     }
     else if (0 == ipd.found) {
         if (tsk_verbose)
-            tsk_printf("File not found: %s\n", ipd.cur_dir);
+            TFPRINTF(stderr, _TSK_T("File not found: %s\n"), tpath);
         *result = 0;
         return 1;
     }
@@ -400,11 +417,11 @@ ifind_data_file_act(TSK_FS_INFO * fs, DADDR_T addr, char *buf,
         (block < (addr + (size + fs->block_size - 1) / fs->block_size))) {
         tsk_printf("%" PRIuINUM "\n", curinode);
 
-        if (!(localflags & TSK_FS_IFIND_ALL)) {
+        if (!(s_localflags & TSK_FS_IFIND_ALL)) {
             fs->close(fs);
             exit(0);
         }
-        found = 1;
+        s_found = 1;
     }
     return TSK_WALK_CONT;
 }
@@ -422,11 +439,11 @@ ifind_data_file_ntfs_act(TSK_FS_INFO * fs, DADDR_T addr, char *buf,
         tsk_printf("%" PRIuINUM "-%" PRIu32 "-%" PRIu16 "\n", curinode,
             curtype, curid);
 
-        if (!(localflags & TSK_FS_IFIND_ALL)) {
+        if (!(s_localflags & TSK_FS_IFIND_ALL)) {
             fs->close(fs);
             exit(0);
         }
-        found = 1;
+        s_found = 1;
     }
     return TSK_WALK_CONT;
 }
@@ -524,7 +541,7 @@ ifind_data_block_act(TSK_FS_INFO * fs, DADDR_T addr, char *buf,
 {
     if (flags & TSK_FS_BLOCK_FLAG_META) {
         tsk_printf("Meta Data\n");
-        found = 1;
+        s_found = 1;
     }
 
     return TSK_WALK_STOP;
@@ -537,8 +554,8 @@ ifind_data_block_act(TSK_FS_INFO * fs, DADDR_T addr, char *buf,
 uint8_t
 tsk_fs_ifind_data(TSK_FS_INFO * fs, uint8_t lclflags, DADDR_T blk)
 {
-    found = 0;
-    localflags = lclflags;
+    s_found = 0;
+    s_localflags = lclflags;
     block = blk;
 
     if (fs->inode_walk(fs, fs->first_inum, fs->last_inum,
@@ -551,7 +568,7 @@ tsk_fs_ifind_data(TSK_FS_INFO * fs, uint8_t lclflags, DADDR_T blk)
      * If we did not find an inode yet, we call block_walk for the 
      * block to find out the associated flags so we can identify it as
      * a meta data block */
-    if (0 == found) {
+    if (0 == s_found) {
         if (fs->block_walk(fs, block, block, (TSK_FS_BLOCK_FLAG_ENUM)
                 (TSK_FS_BLOCK_FLAG_UNALLOC | TSK_FS_BLOCK_FLAG_ALLOC |
                     TSK_FS_BLOCK_FLAG_META | TSK_FS_BLOCK_FLAG_CONT),
@@ -559,7 +576,7 @@ tsk_fs_ifind_data(TSK_FS_INFO * fs, uint8_t lclflags, DADDR_T blk)
             return 1;
         }
     }
-    if (0 == found) {
+    if (0 == s_found) {
         tsk_printf("Inode not found\n");
     }
     return 0;
