@@ -57,6 +57,10 @@
 **        each copy of the program.
 */
 
+/** \file iso9660.h
+ * Contains the structures and function APIs for ISO9660 file system support.
+ */
+
 /* refernece documents used:
  * IEEE P1281 - System Use Sharing Protocol, version 1.12
  * IEEE P1282 - Rock Ridge Interchange Protocol, version 1.12
@@ -92,7 +96,9 @@
 #define ISO9660_RESERVE_CEIL		254
 #define ISO9660_VOL_DESC_SET_TERM	255     /* volume descriptor set terminator */
 
-#define ISO9660_MAXNAMLEN	103     /* joliet max name length */
+#define ISO9660_MAXNAMLEN_STD	128     ///< Maximum length of standard name
+#define ISO9660_MAXNAMLEN_JOL   128     ///< maximum UTF-16 joliet name (in bytes)
+#define ISO9660_MAXNAMLEN   (ISO9660_MAXNAMLEN_JOL << 1)        // mult jol by 2 to account for UTF-16 to UTF-8 conversion
 
 /* Bits in permissions used in extended attribute records.  */
 #define ISO9660_BIT_UR	0x0010
@@ -124,33 +130,8 @@
 #define ISO9660_TYPE_PVD	0
 #define ISO9660_TYPE_SVD	1
 
-/* macros to get numbers.  numbers in ISO9660 are stored in both byte orders.
- * These will grab the one we are using.
- */
-#define parseu16(foo, x) \
-    (uint16_t)( (foo->endian & TSK_LIT_ENDIAN)  ? \
-    ((uint16_t) \
-        ((uint16_t)((uint8_t *)(x))[0] << 0) + \
-        ((uint16_t)((uint8_t *)(x))[1] << 8)) \
-        : \
-    ((uint16_t) \
-        ((uint16_t)((uint8_t *)(x))[2] << 8) + \
-        ((uint16_t)((uint8_t *)(x))[3] << 0)) )
-
-#define parseu32(foo, x) \
-    (uint32_t)( (foo->endian & TSK_LIT_ENDIAN)  ? \
-    ((uint32_t) \
-        ((uint32_t)((uint8_t *)(x))[0] << 0) + \
-        ((uint32_t)((uint8_t *)(x))[1] << 8) + \
-        ((uint32_t)((uint8_t *)(x))[2] << 16) + \
-        ((uint32_t)((uint8_t *)(x))[3] << 24)) \
-        : \
-    ((uint32_t) \
-        ((uint32_t)((uint8_t *)(x))[7] << 0) + \
-        ((uint32_t)((uint8_t *)(x))[6] << 8) + \
-        ((uint32_t)((uint8_t *)(x))[5] << 16) + \
-        ((uint32_t)((uint8_t *)(x))[4] << 24)) )
-
+#define ISO9660_CTYPE_ASCII 0
+#define ISO9660_CTYPE_UTF16 1
 
 /* recording date and time */
 typedef struct {
@@ -179,16 +160,18 @@ typedef struct {
 
 /* iso 9660 directory record */
 typedef struct {
-    uint8_t length;             /* length of directory record */
+    uint8_t entry_len;          /* length of directory record */
     uint8_t ext_len;            /* extended attribute record length */
-    uint8_t ext_loc[8];         /* location of extent (2|32) */
-    uint8_t data_len[8];        /* data length (2|32) */
-    record_data rec;            /* recording date and time */
+    uint8_t ext_loc_l[4];       /* location of extent - le */
+    uint8_t ext_loc_m[4];       /* location of extent - be */
+    uint8_t data_len_l[4];      /* data length - le */
+    uint8_t data_len_m[4];      /* data length - be */
+    record_data rec_time;       /* recording date and time */
     int8_t flags;               /* file flags */
     uint8_t unit_sz;            /* file unit size */
     uint8_t gap_sz;             /* interleave gap size */
     uint8_t seq[4];             /* volume sequence number (2|16) */
-    uint8_t len;                /* length of file identifier */
+    uint8_t fi_len;             /* length of file identifier in bytes */
 } iso9660_dentry;
 
 /* This is a dummy struct used to make reading an entire PVD easier,
@@ -198,8 +181,10 @@ typedef struct {
 typedef struct {
     uint8_t length;             /* length of directory record */
     uint8_t ext_len;            /* extended attribute record length */
-    uint8_t ext_loc[8];         /* location of extent (2|32) */
-    uint8_t data_len[8];        /* data length (2|32) */
+    uint8_t ext_loc_l[4];       /* location of extent - le */
+    uint8_t ext_loc_m[4];       /* location of extent - be */
+    uint8_t data_len_l[4];      /* data length - le */
+    uint8_t data_len_m[4];      /* data length - be */
     record_data rec;            /* recording date and time */
     int8_t flags;               /* file flags */
     uint8_t unit_sz;            /* file unit size */
@@ -211,27 +196,33 @@ typedef struct {
 
 /* generic volume descriptor */
 typedef struct {
-    uint8_t type;               /* volume descriptor type */
-    char magic[ISODCL(2, 6)];   /* magic number. CD001 */
-    char ver[ISODCL(7, 7)];     /* volume descriptor version */
-} iso_vd;
+    uint8_t type;               ///<  volume descriptor type 
+    char magic[ISODCL(2, 6)];   ///< magic number. "CD001" 
+    char ver[ISODCL(7, 7)];     ///< volume descriptor version 
+    char x[ISODCL(8, 2048)];    ///< Depends on descriptor type
+} iso9660_gvd;
 
 /* primary volume descriptor */
 typedef struct {
-    char unused1[ISODCL(8, 8)]; /* should be 0.  unused. */
+    char unused1[ISODCL(1, 8)]; /* should be 0.  unused. */
     char sys_id[ISODCL(9, 40)]; /* system identifier */
     char vol_id[ISODCL(41, 72)];        /* volume identifier */
     char unused2[ISODCL(73, 80)];       /* should be 0.  unused. */
-    uint8_t vol_spc[ISODCL(81, 88)];    /* volume space size (2|32) */
+    uint8_t vs_sz_l[ISODCL(81, 84)];    /* volume space size in blocks - le */
+    uint8_t vs_sz_m[ISODCL(85, 88)];    /* volume space size in blocks - be */
     char unused3[ISODCL(89, 120)];      /* should be 0.  unused. */
-    uint8_t vol_set[ISODCL(121, 124)];  /* volume set size (2|16) */
-    uint8_t vol_seq[ISODCL(125, 128)];  /* volume sequence number (2|16) */
-    uint8_t blk_sz[ISODCL(129, 132)];   /* logical block size (2|16) */
-    uint8_t path_size[ISODCL(133, 140)];        /* path table size (2|32) */
-    uint8_t loc_l[ISODCL(141, 144)];    /* locat of occur of type L path tbl. */
-    uint8_t opt_loc_l[ISODCL(145, 148)];        /* locat of optional occurence */
-    uint8_t loc_m[ISODCL(149, 152)];    /* locat of occur of type M path tbl. */
-    uint8_t opt_loc_m[ISODCL(153, 156)];        /* locat of optional occurence */
+    uint8_t vol_set_l[ISODCL(121, 122)];        /* volume set size - le */
+    uint8_t vol_set_m[ISODCL(123, 124)];        /* volume set size - be */
+    uint8_t vol_seq_l[ISODCL(125, 126)];        /* volume sequence number -le  */
+    uint8_t vol_seq_m[ISODCL(127, 128)];        /* volume sequence number - be */
+    uint8_t blk_sz_l[ISODCL(129, 130)]; /* logical block size - le */
+    uint8_t blk_sz_m[ISODCL(131, 132)]; /* logical block size - be */
+    uint8_t pt_size_l[ISODCL(133, 136)];        /* path table size in bytes - le */
+    uint8_t pt_size_m[ISODCL(137, 140)];        /* path table size in bytes - be  */
+    uint8_t pt_loc_l[ISODCL(141, 144)]; /* log block addr of type L path tbl. */
+    uint8_t pt_opt_loc_l[ISODCL(145, 148)];     /* log block addr of optional L path tbl */
+    uint8_t pt_loc_m[ISODCL(149, 152)]; /* log block addr of type M path tbl. */
+    uint8_t pt_opt_loc_m[ISODCL(153, 156)];     /* log block addr of optional M path tbl */
     iso9660_root_dentry dir_rec;        /* directory record for root dir */
     char vol_setid[ISODCL(191, 318)];   /* volume set identifier */
     unsigned char pub_id[ISODCL(319, 446)];     /* publisher identifier */
@@ -248,24 +239,29 @@ typedef struct {
     char res[ISODCL(883, 883)]; /* reserved */
     char app_use[ISODCL(884, 1395)];    /* application use */
     char reserv[ISODCL(1396, 2048)];    /* reserved */
-} iso_pvd;
+} iso9660_pvd;
 
 /* supplementary volume descriptor */
 typedef struct {
-    uint8_t flags[ISODCL(8, 8)];        /* volume flags */
+    uint8_t flags[ISODCL(1, 8)];        /* volume flags */
     char sys_id[ISODCL(9, 40)]; /* system identifier */
     char vol_id[ISODCL(41, 72)];        /* volume identifier */
     char unused2[ISODCL(73, 80)];       /* should be 0.  unused. */
-    uint8_t vol_spc[ISODCL(81, 88)];    /* volume space size (2|32) */
+    uint8_t vs_sz_l[ISODCL(81, 84)];    /* volume space size in blocks - le */
+    uint8_t vs_sz_m[ISODCL(85, 88)];    /* volume space size in blocks - be */
     uint8_t esc_seq[ISODCL(89, 120)];   /* escape sequences */
-    uint8_t vol_set[ISODCL(121, 124)];  /* volume set size (2|16) */
-    uint8_t vol_seq[ISODCL(125, 128)];  /* volume sequence number (2|16) */
-    uint8_t blk_sz[ISODCL(129, 132)];   /* logical block size (2|16) */
-    uint8_t path_size[ISODCL(133, 140)];        /* path table size (2|32) */
-    uint8_t loc_l[ISODCL(141, 144)];    /* locat of occur of type L path tbl. */
-    uint8_t opt_loc_l[ISODCL(145, 148)];        /* locat of optional occurence */
-    uint8_t loc_m[ISODCL(149, 152)];    /* locat of occur of type M path tbl. */
-    uint8_t opt_loc_m[ISODCL(153, 156)];        /* locat of optional occurence */
+    uint8_t vol_set_l[ISODCL(121, 122)];        /* volume set size - le */
+    uint8_t vol_set_m[ISODCL(123, 124)];        /* volume set size - be */
+    uint8_t vol_seq_l[ISODCL(125, 126)];        /* volume sequence number -le  */
+    uint8_t vol_seq_m[ISODCL(127, 128)];        /* volume sequence number - be */
+    uint8_t blk_sz_l[ISODCL(129, 130)]; /* logical block size - le */
+    uint8_t blk_sz_m[ISODCL(131, 132)]; /* logical block size - be */
+    uint8_t pt_size_l[ISODCL(133, 136)];        /* path table size in bytes - le */
+    uint8_t pt_size_m[ISODCL(137, 140)];        /* path table size in bytes - be  */
+    uint8_t pt_loc_l[ISODCL(141, 144)]; /* log block addr of type L path tbl. */
+    uint8_t pt_opt_loc_l[ISODCL(145, 148)];     /* log block addr of optional type L path tbl. */
+    uint8_t pt_loc_m[ISODCL(149, 152)]; /* log block addr of type M path tbl. */
+    uint8_t pt_opt_loc_m[ISODCL(153, 156)];     /* log block addr of optional type M path tbl. */
     iso9660_root_dentry dir_rec;        /* directory record for root dir */
     char vol_setid[ISODCL(191, 318)];   /* volume set identifier */
     unsigned char pub_id[ISODCL(319, 446)];     /* publisher identifier */
@@ -282,7 +278,7 @@ typedef struct {
     char res[ISODCL(883, 883)]; /* reserved */
     char app_use[ISODCL(884, 1395)];    /* application use */
     char reserv[ISODCL(1396, 2048)];    /* reserved */
-} iso_svd;
+} iso9660_svd;
 
 /* iso 9660 boot record */
 typedef struct {
@@ -318,16 +314,16 @@ typedef struct {
 } ext_attr_rec;
 
 /* primary volume descriptor linked list node */
-typedef struct pvd_node {
-    iso_pvd pvd;
-    struct pvd_node *next;
-} pvd_node;
+typedef struct iso9660_pvd_node {
+    iso9660_pvd pvd;
+    struct iso9660_pvd_node *next;
+} iso9660_pvd_node;
 
 /* supplementary volume descriptor linked list node */
-typedef struct svd_node {
-    iso_svd svd;
-    struct svd_node *next;
-} svd_node;
+typedef struct iso9660_svd_node {
+    iso9660_svd svd;
+    struct iso9660_svd_node *next;
+} iso9660_svd_node;
 
 /* RockRidge extension info */
 typedef struct {
@@ -335,28 +331,32 @@ typedef struct {
     gid_t gid;                  /* group */
     uint16_t mode;              /* posix file mode */
     uint32_t nlink;             /* number of links */
-    char fn[ISO9660_MAXNAMLEN]; /* alternate filename */
+    char fn[ISO9660_MAXNAMLEN_STD];     /* alternate filename */
 } rockridge_ext;
 
-/*
- * Inode
+/**
+ * Internally used structure to hold basic inode information.
+ *
  */
 typedef struct {
     iso9660_dentry dr;          /* directory record */
     ext_attr_rec *ea;           /* extended attribute record */
-    char fn[ISO9660_MAXNAMLEN]; /* file name */
+    char fn[ISO9660_MAXNAMLEN + 1];     /* file name */
     rockridge_ext *rr;          /* RockRidge Extensions */
     int version;
+    OFF_T susp_off;             ///< Byte offset in image of SUSP (or NULL)
+    OFF_T susp_len;             ///< Length in bytes of SUSP
 } iso9660_inode;
 
 /* inode linked list node */
-typedef struct in_node {
+typedef struct iso9660_inode_node {
     iso9660_inode inode;
     OFF_T offset;               /* byte offset of inode into disk */
     INUM_T inum;                /* identifier of inode */
     int size;                   /* kludge: used to flag fifos, etc */
-    struct in_node *next;
-} in_node;
+    int ea_size;                /* length of ext attributes */
+    struct iso9660_inode_node *next;
+} iso9660_inode_node;
 
 /* The all important ISO_INFO struct */
 typedef struct {
@@ -365,9 +365,9 @@ typedef struct {
     iso9660_inode *dinode;      /* cached disk inode */
     uint32_t path_tab_addr;     /* address of path table */
     uint32_t root_addr;         /* address of root dir extent */
-    pvd_node *pvd;              /* primary volume descriptors */
-    svd_node *svd;              /* secondary volume descriptors */
-    in_node *in;                /* list of inodes */
+    iso9660_pvd_node *pvd;      ///< Head of primary volume descriptor list (there should be only one...)
+    iso9660_svd_node *svd;      ///< Head of secondary volume descriptor list 
+    iso9660_inode_node *in_list;        /* list of inodes */
     uint8_t rr_found;           /* 1 if rockridge found */
 } ISO_INFO;
 
@@ -383,25 +383,49 @@ extern uint8_t iso9660_dinode_load(ISO_INFO * iso, INUM_T inum);
  *
  **********************************************************/
 
-/* System use sharing protocol CE entry */
-typedef struct {
-    char sig[ISODCL(1, 2)];     /* signature, should be "CE" */
-    uint8_t len[ISODCL(3, 3)];  /* length of system use entry */
-    uint8_t ver[ISODCL(4, 4)];  /* system use entry version */
-    uint8_t blk_loc[ISODCL(5, 12)];     /* block location */
-    uint8_t offs[ISODCL(13, 20)];       /* offset */
-    uint8_t len_c[ISODCL(21, 28)];      /* length of cont area */
 
-} ce_sys_use;
-
-/* System use  sharing protocol SP entry */
 typedef struct {
-    char sig[ISODCL(1, 2)];     /* signature, should be "SP" */
-    uint8_t len[ISODCL(3, 3)];  /* length of system use entry */
-    uint8_t ver[ISODCL(4, 4)];  /* system use entry version */
-    uint8_t check[ISODCL(5, 6)];        /* check bytes 0xbeef */
-    uint8_t skip[ISODCL(7, 7)]; /* bytes skipped */
-} sp_sys_use;
+    char sig[2];
+    uint8_t len;
+    char ver;
+} iso9660_susp_head;
+
+
+/// SUSP Continuation Entry (CE)
+typedef struct {
+    char sig[2];
+    uint8_t len;
+    char ver;
+    uint8_t blk_l[4];           ///< BLock location of continuation area
+    uint8_t blk_m[4];
+    uint8_t offset_l[4];        ///< Offset to start of continuation area (in bytes)
+    uint8_t offset_m[4];
+    uint8_t celen_l[4];         ///< Length of continuation area (in bytes)
+    uint8_t celen_m[4];
+} iso9660_susp_ce;
+
+/// SUSP SP entry
+typedef struct {
+    char sig[2];
+    uint8_t len;
+    char ver;
+    uint8_t chk[2];
+    uint8_t skip;
+} iso9660_susp_sp;
+
+typedef struct {
+    char sig[2];
+    uint8_t len;
+    char ver;
+    uint8_t len_id;             ///< length of extension id (in bytes)
+    uint8_t len_des;            ///< length of extension desc (in bytes)
+    uint8_t len_src;            ///< Length of extension spec source (in bytes)
+    uint8_t ext_ver;            ///< Version id 
+    char ext_id[1];             ///< Extension ID text (with length of len_id);
+    // next is the extension descriptor text
+    // next is the extension source text
+} iso9660_susp_er;
+
 
 /* Rockridge ISO9660 system use field entry */
 typedef struct {
@@ -414,40 +438,47 @@ typedef struct {
 /* Rockridge PX entry */
 typedef struct {
     char sig[ISODCL(1, 2)];     /* signature, should be "PX" */
-    uint8_t len[ISODCL(3, 3)];  /* length, should be 44 */
-    uint8_t ver[ISODCL(4, 4)];  /* system use entry version (1) */
-    uint8_t mode[ISODCL(5, 12)];        /* POSIX file mode */
-    uint8_t links[ISODCL(13, 20)];      /* POSIX file links */
-    uint8_t uid[ISODCL(21, 28)];        /* POSIX user id */
-    uint8_t gid[ISODCL(29, 36)];        /* POSIX group id */
+    uint8_t len;                /* length, should be 44 */
+    uint8_t ver;                /* system use entry version (1) */
+    uint8_t mode_l[ISODCL(5, 8)];       /* POSIX file mode - le */
+    uint8_t mode_m[ISODCL(9, 12)];      /* POSIX file mode - be */
+    uint8_t links_l[ISODCL(13, 16)];    /* POSIX file links - le */
+    uint8_t links_m[ISODCL(17, 20)];    /* POSIX file links - be */
+    uint8_t uid_l[ISODCL(21, 24)];      /* POSIX user id - le */
+    uint8_t uid_m[ISODCL(25, 28)];      /* POSIX user id - be */
+    uint8_t gid_l[ISODCL(29, 32)];      /* POSIX group id - le */
+    uint8_t gid_m[ISODCL(23, 36)];      /* POSIX group id - be */
     /* rockridge docs say this is here, k3b disagrees... hmmmm */
     //      uint8_t serial[ISODCL(37,44)];  /* POSIX file serial number */
-} rr_px_entry;
+} iso9660_rr_px_entry;
 
 /* Rockridge PN entry */
 typedef struct {
     char sig[ISODCL(1, 2)];     /* signature, should be "PN" */
-    uint8_t len[ISODCL(3, 3)];  /* length, should be 20 */
-    uint8_t ver[ISODCL(4, 4)];  /* system use entry version (1) */
-    uint8_t devt_h[ISODCL(5, 12)];      /* top 32 bits of device # */
-    uint8_t devt_l[ISODCL(13, 20)];     /* low 32 bits of device # */
-} rr_pn_entry;
+    uint8_t len;                /* length, should be 20 */
+    uint8_t ver;                /* system use entry version (1) */
+    uint8_t dev_h_l[ISODCL(5, 8)];      /* top 32 bits of device # */
+    uint8_t dev_h_m[ISODCL(9, 12)];     /* top 32 bits of device # */
+    uint8_t dev_l_l[ISODCL(13, 16)];    /* low 32 bits of device # */
+    uint8_t dev_l_m[ISODCL(17, 20)];    /* low 32 bits of device # */
+} iso9660_rr_pn_entry;
 
 /* Rockridge SL entry */
 typedef struct {
     char sig[ISODCL(1, 2)];     /* signature, should be "SL" */
-    uint8_t len[ISODCL(3, 3)];  /* length */
-    uint8_t ver[ISODCL(4, 4)];  /* system use entry version (1) */
-    uint8_t flags[ISODCL(5, 5)];        /* flags */
-} rr_sl_entry;
+    uint8_t len;                /* length */
+    uint8_t ver;                /* system use entry version (1) */
+    uint8_t flags;              /* flags */
+} iso9660_rr_sl_entry;
 
 /* Rockridge NM entry */
 typedef struct {
     char sig[ISODCL(1, 2)];     /* signature, should be "NM" */
-    uint8_t len[ISODCL(3, 3)];  /* length of alternate name */
+    uint8_t len;                /* length of alternate name */
     uint8_t ver[ISODCL(4, 4)];  /* system use entry version (1) */
     uint8_t flags[ISODCL(5, 5)];        /* flags */
-} rr_nm_entry;
+    char name[1];               // start of the name
+} iso9660_rr_nm_entry;
 
 /* Rockridge CL entry */
 typedef struct {
@@ -455,14 +486,14 @@ typedef struct {
     uint8_t len[ISODCL(3, 3)];  /* length, should be 12 */
     uint8_t ver[ISODCL(4, 4)];  /* system use entry version (1) */
     uint8_t par_loc[ISODCL(5, 12)];     /* location of parent directory */
-} rr_cl_entry;
+} iso9660_rr_cl_entry;
 
 /* Rockridge RE entry */
 typedef struct {
     char sig[ISODCL(1, 2)];     /* signature, should be "RE" */
     uint8_t len[ISODCL(3, 3)];  /* length, should be 4 */
     uint8_t ver[ISODCL(4, 4)];  /* system use entry version (1) */
-} rr_re_entry;
+} iso9660_rr_re_entry;
 
 /* Rockridge TF entry */
 typedef struct {
@@ -470,7 +501,7 @@ typedef struct {
     uint8_t len[ISODCL(3, 3)];  /* length of TF entry */
     uint8_t ver[ISODCL(4, 4)];  /* system use entry version (1) */
     uint8_t flags[ISODCL(5, 5)];        /* flags */
-} rr_tf_entry;
+} iso9660_rr_tf_entry;
 
 /* Rockridge SF entry */
 typedef struct {
@@ -480,6 +511,6 @@ typedef struct {
     uint8_t vfs_h[ISODCL(5, 12)];       /* virtual file size high */
     uint8_t vfs_l[ISODCL(13, 20)];      /* virtual file size low */
     uint8_t depth[ISODCL(21, 21)];      /* table depth */
-} rr_sf_entry;
+} iso9660_rr_sf_entry;
 
 #endif
