@@ -2,7 +2,7 @@
 ** jcat
 ** The Sleuth Kit 
 **
-** $Date: 2007/03/23 15:01:31 $
+** $Date: 2007/08/16 16:14:26 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
@@ -113,6 +113,12 @@ MAIN(int argc, TSK_TCHAR ** argv)
             tsk_error_print(stderr);
             exit(1);
         }
+        if (imgoff >= img->size) {
+            tsk_fprintf(stderr,
+                "Sector offset supplied is larger than disk image (maximum: %"
+                PRIu64 ")\n", img->size / 512);
+            exit(1);
+        }
 
         if ((fs = tsk_fs_open(img, imgoff, fstype)) == NULL) {
             tsk_error_print(stderr);
