@@ -2,7 +2,7 @@
 ** ntfs_dent
 ** The Sleuth Kit
 **
-** $Date: 2007/05/17 19:32:28 $
+** $Date: 2007/11/05 19:08:34 $
 **
 ** name layer support for the NTFS file system
 **
@@ -43,10 +43,15 @@ typedef struct {
     /* The current directory name string */
     char dirs[DIR_STRSZ];
 
+    /* Set to 1 when we are collecting inode allocation information. 
+     * The info is used by inode_walk when looking for ORPHAN files.
+     */
+    uint8_t save_inum_named;
+
 } NTFS_DINFO;
 
 
-static uint8_t
+static TSK_RETVAL_ENUM
 ntfs_dent_walk_lcl(TSK_FS_INFO *, NTFS_DINFO *, TSK_LIST **, INUM_T,
     TSK_FS_DENT_FLAG_ENUM, TSK_FS_DENT_TYPE_WALK_CB, void *);
 /* 
@@ -364,6 +369,7 @@ ntfs_dent_idxentry(NTFS_INFO * ntfs, NTFS_DINFO * dinfo,
             ((fs_dent->fsi->mode & TSK_FS_INODE_MODE_FMT) ==
                 TSK_FS_INODE_MODE_DIR) && (fs_dent->inode)) {
             int depth_added = 0;
+            int retval;
 
             /* Make sure we do not get into an infinite loop */
             if (0 == tsk_list_find(*list_seen, fs_dent->inode)) {
@@ -385,12 +391,22 @@ ntfs_dent_idxentry(NTFS_INFO * ntfs, NTFS_DINFO * dinfo,
                 }
                 dinfo->depth++;
 
-                if (ntfs_dent_walk_lcl(&(ntfs->fs_info), dinfo, list_seen,
-                        fs_dent->inode, flags, action, ptr)) {
-                    if (tsk_verbose)
-                        tsk_fprintf(stderr,
-                            "Error recursing into directory\n");
-                    tsk_error_reset();
+
+                retval =
+                    ntfs_dent_walk_lcl(&(ntfs->fs_info), dinfo, list_seen,
+                    fs_dent->inode, flags, action, ptr);
+                if (retval != TSK_OK) {
+                    if (retval == TSK_COR) {
+                        if (tsk_verbose) {
+                            tsk_fprintf(stderr,
+                                "Error recursing into directory\n");
+                            tsk_error_print(stderr);
+                        }
+                        tsk_error_reset();
+                    }
+                    else {
+                        return 1;
+                    }
                 }
 
                 dinfo->depth--;
@@ -516,11 +532,15 @@ ntfs_fix_idxrec(NTFS_INFO * ntfs, ntfs_idxrec * idxrec, uint32_t len)
 
 
 
-
-
-/* 
- * This function looks up the inode and processes its tree
+/**
+ * Process the contents of a directory and pass each file name to a callback function.
  *
+ * @param fs File system to analyze
+ * @param inum Metadata address of directory to analyze
+ * @param flags Flags used during analysis
+ * @param action Callback function that is called for each file name
+ * @param ptr Pointer to data that is passed to callback
+ * @returns 1 on error and 0 on success
  */
 uint8_t
 ntfs_dent_walk(TSK_FS_INFO * fs, INUM_T inum, TSK_FS_DENT_FLAG_ENUM flags,
@@ -539,18 +559,48 @@ ntfs_dent_walk(TSK_FS_INFO * fs, INUM_T inum, TSK_FS_DENT_FLAG_ENUM flags,
         flags |= (TSK_FS_DENT_FLAG_ALLOC | TSK_FS_DENT_FLAG_UNALLOC);
     }
 
-
     memset(&dinfo, 0, sizeof(NTFS_DINFO));
+
+    /* if the flags are right, we can collect info that may be needed
+     * for an orphan walk.  If the walk fails or stops, the code that
+     * calls the action will clear this stuff. 
+     */
+    if ((fs->list_inum_named == NULL) && (inum == fs->root_inum) &&
+        (flags & (TSK_FS_DENT_FLAG_ALLOC | TSK_FS_DENT_FLAG_UNALLOC |
+                TSK_FS_DENT_FLAG_RECURSE))) {
+        dinfo.save_inum_named = 1;
+    }
+
+
     retval = ntfs_dent_walk_lcl(fs, &dinfo, &list_seen, inum,
         flags, action, ptr);
 
+    /* If there was an error, then we stopped early and we should get
+     * rid of the partial list we were making.
+     */
+    if ((retval != TSK_OK) && (dinfo.save_inum_named == 1)) {
+        tsk_list_free(fs->list_inum_named);
+        fs->list_inum_named = NULL;
+    }
+
     tsk_list_free(list_seen);
     list_seen = NULL;
-    return retval;
+    return (retval == TSK_OK) ? 0 : 1;
 }
 
 
-static uint8_t
+/**
+ * Process an NTFS directory and call the callback for each name entry
+ * in it.
+ *
+ * @param fs File system to analyze
+ * @param dinfo Structure to hold state between calls
+ * @param list_seen List that holds what directories and files were seen.
+ * @param inum Directory number to start analyzing from
+ * @param flags Flags to use while processing
+ * @returns return value 
+ */
+static TSK_RETVAL_ENUM
 ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
     TSK_LIST ** list_seen, INUM_T inum, TSK_FS_DENT_FLAG_ENUM flags,
     TSK_FS_DENT_TYPE_WALK_CB action, void *ptr)
@@ -574,7 +624,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
         tsk_errno = TSK_ERR_FS_WALK_RNG;
         snprintf(tsk_errstr, TSK_ERRSTR_L,
             "ntfs_dent_walk: inode value: %" PRIuINUM "\n", inum);
-        return 1;
+        return TSK_ERR;
     }
 
     if (tsk_verbose)
@@ -587,7 +637,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
     if (!fs_inode) {
         strncat(tsk_errstr2, " - ntfs_dent_walk",
             TSK_ERRSTR_L - strlen(tsk_errstr2));
-        return 1;
+        return TSK_COR;         //this could be an error though...
     }
     if (!fs_inode->attr) {
         tsk_error_reset();
@@ -595,7 +645,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
         snprintf(tsk_errstr, TSK_ERRSTR_L,
             "dent_walk: Error: Directory address %" PRIuINUM
             " has no attributes", inum);
-        return 1;
+        return TSK_COR;
     }
 
 
@@ -610,7 +660,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
         strncat(tsk_errstr2, " - dent_walk: $IDX_ROOT not found",
             TSK_ERRSTR_L - strlen(tsk_errstr2));
         tsk_fs_inode_free(fs_inode);
-        return 1;
+        return TSK_COR;
     }
 
     if (fs_data_root->flags & TSK_FS_DATA_NONRES) {
@@ -619,7 +669,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
         snprintf(tsk_errstr, TSK_ERRSTR_L,
             "dent_walk: $IDX_ROOT is not resident - it should be");
         tsk_fs_inode_free(fs_inode);
-        return 1;
+        return TSK_COR;
     }
     idxroot = (ntfs_idxroot *) fs_data_root->buf;
 
@@ -630,7 +680,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
         snprintf(tsk_errstr, TSK_ERRSTR_L,
             "dent_walk: Attribute type in index root is 0");
         tsk_fs_inode_free(fs_inode);
-        return 1;
+        return TSK_COR;
     }
     else if (tsk_getu32(fs->endian, idxroot->type) != NTFS_ATYPE_FNAME) {
         tsk_error_reset();
@@ -639,7 +689,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
             "ERROR: Directory index is sorted by type: %" PRIu32
             ".\nOnly $FNAME is currently supported", tsk_getu32(fs->endian,
                 idxroot->type));
-        return 1;
+        return TSK_COR;
     }
 
     /* Get the header of the index entry list */
@@ -666,7 +716,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
 
         if ((fs_dent = tsk_fs_dent_alloc(16, 0)) == NULL) {
             tsk_fs_inode_free(fs_inode);
-            return 1;
+            return TSK_ERR;
         }
         /* 
          * "." 
@@ -691,12 +741,12 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
             if (retval == TSK_WALK_STOP) {
                 tsk_fs_dent_free(fs_dent);
                 tsk_fs_inode_free(fs_inode);
-                return 0;
+                return TSK_OK;
             }
             else if (retval == TSK_WALK_ERROR) {
                 tsk_fs_dent_free(fs_dent);
                 tsk_fs_inode_free(fs_inode);
-                return 1;
+                return TSK_ERR;
             }
         }
         else {
@@ -729,12 +779,12 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
                 if (retval == TSK_WALK_STOP) {
                     tsk_fs_dent_free(fs_dent);
                     tsk_fs_inode_free(fs_inode);
-                    return 0;
+                    return TSK_OK;
                 }
                 else if (retval == TSK_WALK_ERROR) {
                     tsk_fs_dent_free(fs_dent);
                     tsk_fs_inode_free(fs_inode);
-                    return 1;
+                    return TSK_ERR;
                 }
             }
             else {
@@ -770,7 +820,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
             "Error: Index list offsets are invalid on entry: %" PRIuINUM,
             fs_inode->addr);
         tsk_fs_inode_free(fs_inode);
-        return 1;
+        return TSK_COR;
     }
 
     retval = ntfs_dent_idxentry(ntfs, dinfo, list_seen, idxe,
@@ -781,7 +831,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
 
     if (retval != 0) {
         tsk_fs_inode_free(fs_inode);
-        return (retval == -1) ? 1 : 0;
+        return (retval == -1) ? TSK_ERR : TSK_OK;
     }
 
     /* 
@@ -802,10 +852,10 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
             tsk_errno = TSK_ERR_FS_INODE_INT;
             snprintf(tsk_errstr, TSK_ERRSTR_L,
                 "Error: $IDX_ROOT says there should be children, but there isn't");
-            retval = 1;
+            retval = TSK_COR;
         }
         else {
-            retval = 0;
+            retval = TSK_OK;
         }
         tsk_fs_inode_free(fs_inode);
         return retval;
@@ -818,7 +868,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
         snprintf(tsk_errstr, TSK_ERRSTR_L,
             "$IDX_ALLOC is Resident - it shouldn't be");
         tsk_fs_inode_free(fs_inode);
-        return 1;
+        return TSK_COR;
     }
 
     /* 
@@ -827,7 +877,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
     idxalloc_len = tsk_fs_data_alloc->allocsize;
     if ((idxalloc = tsk_malloc((size_t) idxalloc_len)) == NULL) {
         tsk_fs_inode_free(fs_inode);
-        return 1;
+        return TSK_ERR;
     }
 
     /* Fill in the loading data structure */
@@ -845,7 +895,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
         tsk_fs_inode_free(fs_inode);
         strncat(tsk_errstr2, " - ntfs_dent_walk",
             TSK_ERRSTR_L - strlen(tsk_errstr2));
-        return 1;
+        return TSK_COR;         // this could be an error though
     }
 
     /* Not all of the directory was copied, so we exit */
@@ -857,7 +907,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
         tsk_errno = TSK_ERR_FS_FWALK;
         snprintf(tsk_errstr, TSK_ERRSTR_L,
             "Error reading directory contents: %" PRIuINUM "\n", inum);
-        return 1;
+        return TSK_COR;
     }
 
     /*
@@ -915,7 +965,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
         if (ntfs_fix_idxrec(ntfs, idxrec_p, rec_len)) {
             free(idxalloc);
             tsk_fs_inode_free(fs_inode);
-            return 1;
+            return TSK_COR;
         }
 
         /* Locate the start of the index entry list */
@@ -942,7 +992,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
                 PRIuINUM, fs_inode->addr);
             tsk_fs_inode_free(fs_inode);
             free(idxalloc);
-            return 1;
+            return TSK_COR;
         }
 
 
@@ -954,7 +1004,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
         if (retval != 0) {
             tsk_fs_inode_free(fs_inode);
             free(idxalloc);
-            return (retval == -1) ? 1 : 0;
+            return (retval == -1) ? TSK_ERR : TSK_OK;
         }
 
         /* reset the pointer to the next record */
@@ -981,7 +1031,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
         if (ntfs_fix_idxrec(ntfs, idxrec_p, rec_len)) {
             tsk_fs_inode_free(fs_inode);
             free(idxalloc);
-            return 1;
+            return TSK_COR;
         }
 
         idxelist = &idxrec_p->list;
@@ -1005,7 +1055,7 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
                 PRIuINUM, fs_inode->addr);
             tsk_fs_inode_free(fs_inode);
             free(idxalloc);
-            return 1;
+            return TSK_COR;
         }
 
         /* process the list of index entries */
@@ -1016,14 +1066,16 @@ ntfs_dent_walk_lcl(TSK_FS_INFO * fs, NTFS_DINFO * dinfo,
         if (retval != 0) {
             tsk_fs_inode_free(fs_inode);
             free(idxalloc);
-            return (retval == -1) ? 1 : 0;
+            return (retval == -1) ? TSK_ERR : TSK_OK;
         }
     }
 
     tsk_fs_inode_free(fs_inode);
     free(idxalloc);
 
-    return 0;
+    return TSK_OK;
+    // DOUBLY CHECK RETURN VALUE S
+
 }                               /* end of dent_walk */
 
 
