@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit
 **
-** $Date: 2007/04/04 20:06:58 $
+** $Date: 2007/08/29 19:20:04 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
@@ -15,6 +15,12 @@
 #include "fs_tools_i.h"
 
 
+/**
+ *\file rawfs.c
+ * General "raw" file system functions.  The raw file system is used to process 
+ * an arbitrary chunk of data as 512-byte sectors that have no other structure.
+ * This means that you can use the data-level tools, but that is it. 
+ */
 
 /**************************************************************************
  *
@@ -107,7 +113,7 @@ rawfs_block_walk(TSK_FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk,
 
         cnt = tsk_fs_read_block(fs, data_buf, fs->block_size, addr);
         if (cnt != fs->block_size) {
-            if (cnt != -1) {
+            if (cnt >= 0) {
                 tsk_error_reset();
                 tsk_errno = TSK_ERR_FS_READ;
             }
@@ -174,8 +180,13 @@ rawfs_dent_walk(TSK_FS_INFO * fs, INUM_T inode,
 }
 
 
-/*
- * return 1 on error and 0 on success
+/**
+ * Print details about the file system to a file handle. 
+ *
+ * @param fs File system to print details on
+ * @param hFile File handle to print text to
+ * 
+ * @returns 1 on error and 0 on success
  */
 static uint8_t
 rawfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
@@ -190,8 +201,16 @@ rawfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
 
 /************************* istat *******************************/
 
-/*
- * return 1 on error and 0 on success
+/**
+ * Print details on a specific file to a file handle. 
+ *
+ * @param fs File system file is located in
+ * @param hFile File handle to print text to
+ * @param inum Address of file in file system
+ * @param numblock The number of blocks in file to force print (can go beyond file size)
+ * @param sec_skew Clock skew in seconds to also print times in
+ * 
+ * @returns 1 on error and 0 on success
  */
 static uint8_t
 rawfs_istat(TSK_FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
@@ -253,10 +272,14 @@ rawfs_close(TSK_FS_INFO * fs)
 }
 
 
-/* rawfs_open - open a file as raw 
+/**
+ * Open part of a disk image as a raw file system -- which basically means that it has no file system structure.
+ * The data is considered to be in 512-byte sectors. 
  *
- * Return NULL on error
- * */
+ * @param img_info Disk image to analyze
+ * @param offset Byte offset where "file system" starts
+ * @returns NULL on error
+ */
 TSK_FS_INFO *
 rawfs_open(TSK_IMG_INFO * img_info, SSIZE_T offset)
 {
@@ -290,7 +313,7 @@ rawfs_open(TSK_IMG_INFO * img_info, SSIZE_T offset)
         fs->block_count++;
 
     fs->first_block = 0;
-    fs->last_block = fs->block_count - 1;
+    fs->last_block = fs->last_block_act = fs->block_count - 1;
     fs->block_size = 512;
     fs->dev_bsize = 512;
 
