@@ -1,7 +1,7 @@
 /*
 ** The Sleuth Kit 
 **
-** $Date: 2007/04/04 20:06:58 $
+** $Date: 2007/08/29 19:20:04 $
 **
 ** Brian Carrier [carrier@sleuthkit.org]
 ** Copyright (c) 2006 Brian Carrier, Basis Technology.  All Rights reserved
@@ -14,6 +14,14 @@
 
 #include "fs_tools_i.h"
 
+/**
+ *\file swapfs.c
+ * General "swapfs" file system functions.  The "swap" file system is used to process 
+ * an arbitrary chunk of data as 4096-byte pages that have no other structure.
+ * This means that you can use the data-level tools, but that is it.  This is similar to
+ * the rawfs code, but a different block size. This is primarily intended for Unix systems
+ * that have a swap space partition. 
+ */
 
 /**************************************************************************
  *
@@ -106,7 +114,7 @@ swapfs_block_walk(TSK_FS_INFO * fs, DADDR_T start_blk, DADDR_T end_blk,
 
         cnt = tsk_fs_read_block(fs, data_buf, fs->block_size, addr);
         if (cnt != fs->block_size) {
-            if (cnt != -1) {
+            if (cnt >= 0) {
                 tsk_error_reset();
                 tsk_errno = TSK_ERR_FS_READ;
             }
@@ -174,8 +182,13 @@ swapfs_dent_walk(TSK_FS_INFO * fs, INUM_T inode,
 }
 
 
-/*
- * return 1 on error and 0 on success
+/**
+ * Print details about the file system to a file handle. 
+ *
+ * @param fs File system to print details on
+ * @param hFile File handle to print text to
+ * 
+ * @returns 1 on error and 0 on success
  */
 static uint8_t
 swapfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
@@ -189,7 +202,17 @@ swapfs_fsstat(TSK_FS_INFO * fs, FILE * hFile)
 
 /************************* istat *******************************/
 
-/* Return 1 on error and 0 on success */
+/**
+ * Print details on a specific file to a file handle. 
+ *
+ * @param fs File system file is located in
+ * @param hFile File handle to print text to
+ * @param inum Address of file in file system
+ * @param numblock The number of blocks in file to force print (can go beyond file size)
+ * @param sec_skew Clock skew in seconds to also print times in
+ * 
+ * @returns 1 on error and 0 on success
+ */
 static uint8_t
 swapfs_istat(TSK_FS_INFO * fs, FILE * hFile, INUM_T inum, DADDR_T numblock,
     int32_t sec_skew)
@@ -245,10 +268,14 @@ swapfs_close(TSK_FS_INFO * fs)
 }
 
 
-/* swaps_open - open a swap file
- * Return TSK_FS_INFO on success and NULL on error
- * */
-
+/**
+ * Open part of a disk image as "swap" space.  This assumes no structure exists. 
+ * Data are organized into 4096-byte pages.
+ *
+ * @param img_info Disk image to analyze
+ * @param offset Byte offset where swap space starts.
+ * @returns NULL on error 
+ */
 TSK_FS_INFO *
 swapfs_open(TSK_IMG_INFO * img_info, SSIZE_T offset)
 {
@@ -281,7 +308,7 @@ swapfs_open(TSK_IMG_INFO * img_info, SSIZE_T offset)
         fs->block_count++;
 
     fs->first_block = 0;
-    fs->last_block = fs->block_count - 1;
+    fs->last_block = fs->last_block_act = fs->block_count - 1;
     fs->block_size = 4096;
     fs->dev_bsize = 512;
 
