@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2007/04/20 21:49:31 $
+ * $Date: 2007/11/14 14:37:57 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2003-2007 Brian Carrier.  All rights reserved
@@ -264,7 +264,7 @@ hk_makeindex(TSK_HDB_INFO * hdb_info, TSK_TCHAR * dbtype)
     size_t len = 0;
     char buf[TSK_HDB_MAXLEN];
     char *hash = NULL, phash[TSK_HDB_HTYPE_MD5_LEN + 1];
-    off_t offset = 0;
+    OFF_T offset = 0;
     int db_cnt = 0, idx_cnt = 0, ig_cnt = 0;
 
     if (tsk_hdb_idxinitialize(hdb_info, dbtype)) {
@@ -284,7 +284,7 @@ hk_makeindex(TSK_HDB_INFO * hdb_info, TSK_TCHAR * dbtype)
     /* read each line of the file */
     fseek(hdb_info->hDb, 0, SEEK_SET);
     for (i = 0; NULL != fgets(buf, TSK_HDB_MAXLEN, hdb_info->hDb);
-         offset += (off_t) len, i++) {
+         offset += (OFF_T) len, i++) {
 
         len = strlen(buf);
 
