@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2007/04/05 18:29:36 $
+ * $Date: 2007/06/05 20:04:41 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2003-2007 Brian Carrier.  All rights reserved
@@ -228,8 +228,7 @@ tsk_hdb_idxinitialize(TSK_HDB_INFO * hdb_info, TSK_TCHAR * htype)
     default:
         tsk_error_reset();
         tsk_errno = TSK_ERR_HDB_CREATE;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-                 "idxinit: Invalid db type\n");
+        snprintf(tsk_errstr, TSK_ERRSTR_L, "idxinit: Invalid db type\n");
         return 1;
     }
 
@@ -276,6 +275,12 @@ tsk_hdb_idxfinalize(TSK_HDB_INFO * hdb_info)
     fclose(hdb_info->hIdxTmp);
     hdb_info->hIdxTmp = NULL;
 
+    /* Close the existing index if it is open */
+    if (hdb_info->hIdx) {
+        fclose(hdb_info->hIdx);
+        hdb_info->hIdx = NULL;
+    }
+
     if (tsk_verbose)
         tsk_fprintf(stderr, "hdb_idxfinalize: Sorting index\n");
 
@@ -328,9 +333,16 @@ tsk_hdb_idxfinalize(TSK_HDB_INFO * hdb_info)
     if (tsk_verbose)
         tsk_fprintf(stderr, "hdb_idxfinalize: Sorting index\n");
 
+    /* Close the unsorted file */
     fclose(hdb_info->hIdxTmp);
     hdb_info->hIdxTmp = NULL;
 
+    /* Close the existing index if it is open */
+    if (hdb_info->hIdx) {
+        fclose(hdb_info->hIdx);
+        hdb_info->hIdx = NULL;
+    }
+
     if (0 == stat(local, &stats)) {
         snprintf(buf, TSK_HDB_MAXLEN, "%s -o %s %s", local,
                  hdb_info->idx_fname, hdb_info->uns_fname);
@@ -364,21 +376,6 @@ tsk_hdb_idxfinalize(TSK_HDB_INFO * hdb_info)
     return 0;
 }
 
-/* Give the offset and the length of a line, this returns the 
- * address of the begining of the line
- */
-/**
- * Determine the starting address of the line in the index that
- * contains the offset.
- * 
- * @param off Byte offset to find start of
- * @param len Byte length of each line in index
- *
- * @return Byte offset to start of corresponding line
- */
-#define base_addr(off, len) \
-  ((len) * ( ( ( (off) + (len) - 1) / (len) ) - 1 ) )
-
 
 /** \internal
  * Setup the internal variables to read an index. This
@@ -489,7 +486,8 @@ hdb_setupindex(TSK_HDB_INFO * hdb_info, uint8_t htype)
         return 1;
     }
 
-    if (strncmp(head, TSK_HDB_IDX_HEAD_STR, strlen(TSK_HDB_IDX_HEAD_STR)) != 0) {
+    if (strncmp(head, TSK_HDB_IDX_HEAD_STR, strlen(TSK_HDB_IDX_HEAD_STR))
+        != 0) {
         tsk_error_reset();
         tsk_errno = TSK_ERR_HDB_UNKTYPE;
         snprintf(tsk_errstr, TSK_ERRSTR_L,
@@ -630,7 +628,7 @@ tsk_hdb_lookup(TSK_HDB_INFO * hdb_info, char *hash,
 
 
     /* See if we have had a lookup yet -- and therefore initialized the variables */
-    if (hdb_info->idx_size == 0) {
+    if (hdb_info->hIdx == NULL) {
         if (hdb_setupindex(hdb_info, htype))
             return -1;
     }
@@ -659,7 +657,7 @@ tsk_hdb_lookup(TSK_HDB_INFO * hdb_info, char *hash,
         }
 
         /* Get the middle of the windows that we are looking at */
-        offset = base_addr(((up - low) / 2), hdb_info->idx_llen);
+        offset = rounddown(((up - low) / 2), hdb_info->idx_llen);
 
         /* Sanity Check */
         if ((offset % hdb_info->idx_llen) != 0) {
@@ -679,12 +677,12 @@ tsk_hdb_lookup(TSK_HDB_INFO * hdb_info, char *hash,
         }
 
         /* Seek to the offset and read it */
-        if (0 != fseek(hdb_info->hIdx, (long) offset, SEEK_SET)) {
+        if (0 != fseeko(hdb_info->hIdx, offset, SEEK_SET)) {
             tsk_error_reset();
             tsk_errno = TSK_ERR_HDB_READIDX;
             snprintf(tsk_errstr, TSK_ERRSTR_L,
-                     "hdb_lookup: Error seeking in search: %lu",
-                     (unsigned long) offset);
+                     "hdb_lookup: Error seeking in search: %" PRIuOFF,
+                     offset);
             return -1;
         }
 
@@ -741,9 +739,14 @@ tsk_hdb_lookup(TSK_HDB_INFO * hdb_info, char *hash,
             else {
                 OFF_T tmpoff, db_off;
 
+#ifdef TSK_WIN32
                 db_off =
-                    (off_t) strtoul(hdb_info->idx_lbuf +
-                                    hdb_info->hash_len + 1, NULL, 10);
+                    _atoi64(&hdb_info->idx_lbuf[hdb_info->hash_len + 1]);
+#else
+                db_off =
+                    strtoull(&hdb_info->idx_lbuf[hdb_info->hash_len + 1],
+                             NULL, 10);
+#endif
 
                 /* Print the one that we found first */
                 if (hdb_info->
@@ -765,13 +768,12 @@ tsk_hdb_lookup(TSK_HDB_INFO * hdb_info, char *hash,
                     if (tmpoff <= 0)
                         break;
 
-                    if (0 !=
-                        fseek(hdb_info->hIdx, (long) tmpoff, SEEK_SET)) {
+                    if (0 != fseeko(hdb_info->hIdx, tmpoff, SEEK_SET)) {
                         tsk_error_reset();
                         tsk_errno = TSK_ERR_HDB_READIDX;
                         snprintf(tsk_errstr, TSK_ERRSTR_L,
-                                 "hdb_lookup: Error seeking for prev entries: %lu",
-                                 (unsigned long) tmpoff);
+                                 "hdb_lookup: Error seeking for prev entries: %"
+                                 PRIuOFF, tmpoff);
                         return -1;
                     }
 
@@ -801,9 +803,17 @@ tsk_hdb_lookup(TSK_HDB_INFO * hdb_info, char *hash,
                         break;
                     }
 
+#ifdef TSK_WIN32
                     db_off =
-                        (off_t) strtoul(hdb_info->idx_lbuf +
-                                        hdb_info->hash_len + 1, NULL, 10);
+                        _atoi64(&hdb_info->
+                                idx_lbuf[hdb_info->hash_len + 1]);
+#else
+
+                    db_off =
+                        strtoull(&hdb_info->
+                                 idx_lbuf[hdb_info->hash_len + 1], NULL,
+                                 10);
+#endif
                     if (hdb_info->
                         getentry(hdb_info, hash, db_off, flags, action,
                                  ptr)) {
@@ -816,13 +826,12 @@ tsk_hdb_lookup(TSK_HDB_INFO * hdb_info, char *hash,
                 tmpoff = offset + hdb_info->idx_llen;
                 while (tmpoff < up) {
 
-                    if (0 !=
-                        fseek(hdb_info->hIdx, (long) tmpoff, SEEK_SET)) {
+                    if (0 != fseeko(hdb_info->hIdx, tmpoff, SEEK_SET)) {
                         tsk_error_reset();
                         tsk_errno = TSK_ERR_HDB_READIDX;
                         snprintf(tsk_errstr, TSK_ERRSTR_L,
-                                 "hdb_lookup: Error seeking for next entries: %lu",
-                                 (unsigned long) tmpoff);
+                                 "hdb_lookup: Error seeking for next entries: %"
+                                 PRIuOFF, tmpoff);
                         return -1;
                     }
 
@@ -853,10 +862,16 @@ tsk_hdb_lookup(TSK_HDB_INFO * hdb_info, char *hash,
                     if (strcasecmp(hdb_info->idx_lbuf, hash) != 0) {
                         break;
                     }
-
+#ifdef TSK_WIN32
                     db_off =
-                        (off_t) strtoul(hdb_info->idx_lbuf +
-                                        hdb_info->hash_len + 1, NULL, 10);
+                        _atoi64(&hdb_info->
+                                idx_lbuf[hdb_info->hash_len + 1]);
+#else
+                    db_off =
+                        strtoull(&hdb_info->
+                                 idx_lbuf[hdb_info->hash_len + 1], NULL,
+                                 10);
+#endif
                     if (hdb_info->
                         getentry(hdb_info, hash, db_off, flags, action,
                                  ptr)) {
@@ -1022,7 +1037,7 @@ tsk_hdb_open(TSK_TCHAR * db_file, TSK_HDB_OPEN_ENUM flags)
                      "hdb_open: Error determining DB type");
             return NULL;
         }
-        fseek(hDb, 0, SEEK_SET);
+        fseeko(hDb, 0, SEEK_SET);
     }
     else {
         dbtype = TSK_HDB_DBTYPE_IDXONLY_ID;
@@ -1117,7 +1132,8 @@ tsk_hdb_close(TSK_HDB_INFO * hdb_info)
     if (hdb_info->idx_fname)
         free(hdb_info->idx_fname);
 
-    fclose(hdb_info->hDb);
+    if (hdb_info->hDb)
+        fclose(hdb_info->hDb);
 
     free(hdb_info);
 }
