@@ -1,5 +1,5 @@
 /*
- * $Date: 2007/06/05 20:04:41 $
+ * $Date: 2007/11/14 14:37:57 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006-2007 Brian Carrier, Basis Technology.  All rights reserved
@@ -27,16 +27,23 @@ aff_read_random(TSK_IMG_INFO * img_info, OFF_T vol_offset, char *buf,
 {
     SSIZE_T cnt;
     IMG_AFF_INFO *aff_info = (IMG_AFF_INFO *) img_info;
-    off_t tot_offset = offset + vol_offset;
+    OFF_T tot_offset = offset + vol_offset;
 
     if (tsk_verbose)
         tsk_fprintf(stderr,
             "aff_read_random: byte offset: %" PRIuOFF " len: %" PRIuOFF
             "\n", offset, len);
 
+    if (tot_offset > img_info->size) {
+        tsk_error_reset();
+        tsk_errno = TSK_ERR_IMG_READ_OFF;
+        snprintf(tsk_errstr, TSK_ERRSTR_L,
+            "aff_read_random - %" PRIuOFF, tot_offset);
+        return -1;
+    }
 
     if (aff_info->seek_pos != tot_offset) {
-        if ((off_t) af_seek(aff_info->af_file, tot_offset, SEEK_SET) !=
+        if (af_seek(aff_info->af_file, tot_offset, SEEK_SET) !=
             tot_offset) {
             tsk_error_reset();
             // @@@ ADD more specific error messages
@@ -61,6 +68,18 @@ aff_read_random(TSK_IMG_INFO * img_info, OFF_T vol_offset, char *buf,
         return -1;
     }
 
+    /* AFF will return 0 if the page does not exist -- fill the 
+     * buffer with zeros in this case */
+    if (cnt == 0) {
+        // @@@ We could improve this if there is an AFF call
+        // to see if the data exists or not
+        if ((af_eof(aff_info->af_file) == 0) &&
+            (tot_offset + len < img_info->size)) {
+            memset(buf, 0, len);
+            cnt = len;
+        }
+    }
+
     aff_info->seek_pos += cnt;
     return cnt;
 }
@@ -207,6 +226,7 @@ aff_close(TSK_IMG_INFO * img_info)
 {
     IMG_AFF_INFO *aff_info = (IMG_AFF_INFO *) img_info;
     af_close(aff_info->af_file);
+    free(aff_info);
 }
 
 
