@@ -12,7 +12,7 @@ typedef struct IMG_AFF_INFO IMG_AFF_INFO;
 struct IMG_AFF_INFO {
     TSK_IMG_INFO img_info;
     AFFILE *af_file;
-    off_t seek_pos;
+    OFF_T seek_pos;
     uint16_t type;              /* TYPE - uses AF_IDENTIFY_x values */
 };
 
