@@ -1,5 +1,5 @@
 /*
- * $Date: 2007/06/05 20:04:41 $
+ * $Date: 2007/11/05 18:22:11 $
  *
  * Joachim Metz <forensics@hoffmannbv.nl>, Hoffmann Investigations
  * Copyright (c) 2006 Joachim Metz.  All rights reserved 
@@ -28,6 +28,14 @@ ewf_image_read_random(TSK_IMG_INFO * img_info, OFF_T vol_offset, char *buf,
             "ewf_read_random: byte offset: %" PRIuOFF " len: %" PRIuOFF
             "\n", offset, len);
 
+    if (tot_offset > img_info->size) {
+        tsk_error_reset();
+        tsk_errno = TSK_ERR_IMG_READ_OFF;
+        snprintf(tsk_errstr, TSK_ERRSTR_L,
+            "split_read_random - %" PRIuOFF, tot_offset);
+        return -1;
+    }
+
     cnt = libewf_read_random(ewf_info->handle, buf, len, tot_offset);
     if (cnt < 0) {
         tsk_error_reset();
