@@ -2,7 +2,7 @@
  * img_cat
  * The Sleuth Kit 
  *
- * $Date: 2007/03/20 21:54:53 $
+ * $Date: 2007/08/29 19:20:04 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved 
@@ -100,7 +100,7 @@ MAIN(int argc, TSK_TCHAR ** argv)
 
         cnt = img->read_random(img, 0, buf, len, done);
         if (cnt != len) {
-            if (cnt != -1) {
+            if (cnt >= 0) {
                 tsk_fprintf(stderr,
                     "img_cat: Error reading image file at offset: %"
                     PRIuOFF ", len: %" PRIuOFF ", return: %" PRIuOFF "\n",
