@@ -1,5 +1,5 @@
 /*
- * $Date: 2007/06/05 20:04:41 $
+ * $Date: 2007/11/05 18:22:11 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006-2007 Brian Carrier, Basis Technology.  All rights reserved
@@ -17,6 +17,10 @@
  * Code to open and read single raw disk images
  */
 
+#if defined(__APPLE__)
+#include <sys/disk.h>
+#endif
+
 #include <sys/stat.h>
 #include "img_tools.h"
 #include "raw.h"
@@ -25,7 +29,20 @@
 #include "Winioctl.h"
 #endif
 
-/* Return the size read and -1 if error */
+
+/**
+ * Read an arbitrary amount of data from a specific location in a raw image file.
+ * This takes two offsets are arguments.  The first is the offset of the volume in the
+ * image file and the second is the offset in the volume.  Both are added to find the 
+ * actual offset.
+ *
+ * @param img_info The image to read from.
+ * @param vol_offset The byte offset into the image file of the volume being read.
+ * @param buf [out] Buffer to store data in
+ * @param len Number of bytes to read
+ * @param offset Byte offset into the volume to start reading from.
+ * @returns The number of bytes read or -1 on error -- which can occur if the offset is larger than the img.
+ */
 static SSIZE_T
 raw_read_random(TSK_IMG_INFO * img_info, OFF_T vol_offset, char *buf,
     OFF_T len, OFF_T offset)
@@ -48,6 +65,14 @@ raw_read_random(TSK_IMG_INFO * img_info, OFF_T vol_offset, char *buf,
     else {
         OFF_T tot_offset = offset + vol_offset;
 
+        if (tot_offset > img_info->size) {
+            tsk_error_reset();
+            tsk_errno = TSK_ERR_IMG_READ_OFF;
+            snprintf(tsk_errstr, TSK_ERRSTR_L,
+                "raw_read_random - %" PRIuOFF, tot_offset);
+            return -1;
+        }
+
 #ifdef TSK_WIN32
         {
             DWORD nread;
@@ -272,6 +297,21 @@ raw_open(const TSK_TCHAR ** images, TSK_IMG_INFO * next)
          * devices and such */
         img_info->size = lseek(raw_info->fd, 0, SEEK_END);
         lseek(raw_info->fd, 0, SEEK_SET);
+
+#if defined(__APPLE__)
+        /* OS X doesn't support SEEK_END on devices */
+        if (img_info->size == 0) {
+            int blkSize;
+            long long blkCnt;
+
+            if (ioctl(raw_info->fd, DKIOCGETBLOCKSIZE, &blkSize) >= 0) {
+                if (ioctl(raw_info->fd, DKIOCGETBLOCKCOUNT, &blkCnt) >= 0) {
+                    img_info->size = blkCnt * (long long) blkSize;
+                }
+            }
+        }
+#endif                          // apple
+
 #endif
         raw_info->seek_pos = 0;
     }
