@@ -1,5 +1,5 @@
 /*
- * $Date: 2007/06/05 20:04:41 $
+ * $Date: 2007/11/05 18:22:12 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006-2007 Brian Carrier, Basis Technology.  All rights reserved
@@ -181,6 +181,14 @@ split_read_random(TSK_IMG_INFO * img_info, OFF_T vol_offset, char *buf,
     // Find the offset of the data
     tot_offset = offset + vol_offset;
 
+    if (tot_offset > img_info->size) {
+        tsk_error_reset();
+        tsk_errno = TSK_ERR_IMG_READ_OFF;
+        snprintf(tsk_errstr, TSK_ERRSTR_L,
+            "split_read_random - %" PRIuOFF, tot_offset);
+        return -1;
+    }
+
     // Find the location of the offset
     for (i = 0; i < split_info->num_img; i++) {
 
