@@ -1,22 +1,30 @@
 # Script for compiling libewf
 # as part of The Sleuth Kit
 #
-# $Date: 2007/05/17 20:05:08 $
+# $Date: 2007/09/06 20:47:21 $
 #
 cd libewf
 if (test -f './Makefile') then
     true
 else
     (touch Makefile.am && \
+     touch libewf/Makefile.am && \
+     touch ewftools/Makefile.am && \
+     touch configure.ac && \
      touch configure.in && \
      touch aclocal.m4 && \
      touch Makefile.in && \
+     touch libewf/Makefile.in && \
      touch config.h.in && \
+     touch libewf/libewf_config.h.in && \
+     touch include/libewf.h.in && \
+     touch include/libewf/libewf_definitions.h.in && \
+     touch include/libewf/libewf_types.h.in && \
      touch configure) || (cd ..; exit $?)
     ./configure --enable-shared=no || (cd ..; exit $?)
 fi
 
-make -C libewf
+make library
 
 if (test -f 'libewf/.libs/libewf.a') then 
     mkdir libewf-tsk
