@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2007/04/04 18:48:46 $
+ * $Date: 2007/08/29 19:20:04 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
@@ -691,7 +691,7 @@ dos_load_ext_table(TSK_MM_INFO * mm, DADDR_T sect_cur,
     cnt = tsk_mm_read_block_nobuf
         (mm, (char *) &sect, sizeof(sect), sect_cur);
     if (cnt != sizeof(sect)) {
-        if (cnt != -1) {
+        if (cnt >= 0) {
             tsk_error_reset();
             tsk_errno = TSK_ERR_MM_READ;
         }
@@ -835,7 +835,7 @@ dos_load_prim_table(TSK_MM_INFO * mm, uint8_t test)
         (mm, (char *) &sect, sizeof(sect), DOS_PART_SOFFSET);
 
     if (cnt != sizeof(sect)) {
-        if (cnt != -1) {
+        if (cnt >= 0) {
             tsk_error_reset();
             tsk_errno = TSK_ERR_MM_READ;
         }
