@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2007/04/04 18:48:46 $
+ * $Date: 2007/08/29 19:20:04 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
@@ -45,7 +45,7 @@ gpt_load_table(TSK_MM_INFO * mm)
         (mm, (char *) &dos_part, sizeof(dos_part), GPT_PART_SOFFSET);
     /* if -1, then tsk_errno is already set */
     if (cnt != sizeof(dos_part)) {
-        if (cnt != -1) {
+        if (cnt >= 0) {
             tsk_error_reset();
             tsk_errno = TSK_ERR_MM_READ;
         }
@@ -87,7 +87,7 @@ gpt_load_table(TSK_MM_INFO * mm)
     cnt = tsk_mm_read_block_nobuf
         (mm, (char *) &head, sizeof(head), GPT_PART_SOFFSET + 1);
     if (cnt != sizeof(head)) {
-        if (cnt != -1) {
+        if (cnt >= 0) {
             tsk_error_reset();
             tsk_errno = TSK_ERR_MM_READ;
         }
@@ -151,7 +151,7 @@ gpt_load_table(TSK_MM_INFO * mm)
         cnt = tsk_mm_read_block_nobuf(mm, ent_buf, mm->block_size,
             tsk_getu64(mm->endian, &head.tab_start_lba) + a);
         if (cnt != mm->block_size) {
-            if (cnt != -1) {
+            if (cnt >= 0) {
                 tsk_error_reset();
                 tsk_errno = TSK_ERR_MM_READ;
             }
