@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2007/04/04 18:48:46 $
+ * $Date: 2007/08/29 19:20:04 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2006 Brian Carrier, Basis Technology.  All rights reserved
@@ -57,7 +57,7 @@ mac_load_table(TSK_MM_INFO * mm)
 
         /* If -1, then tsk_errno is already set */
         if (cnt != sizeof(part)) {
-            if (cnt != -1) {
+            if (cnt >= 0) {
                 tsk_error_reset();
                 tsk_errno = TSK_ERR_MM_READ;
             }
