@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2007/04/04 18:48:47 $
+ * $Date: 2007/08/16 16:14:17 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2005 Brian Carrier.  All rights reserved
@@ -105,6 +105,12 @@ MAIN(int argc, TSK_TCHAR ** argv)
         tsk_error_print(stderr);
         exit(1);
     }
+    if (imgoff >= img->size) {
+        tsk_fprintf(stderr,
+            "Sector offset supplied is larger than disk image (maximum: %"
+            PRIu64 ")\n", img->size / 512);
+        exit(1);
+    }
 
 
     /* process the partition tables */
