@@ -518,7 +518,11 @@ static void
 print_strings (const char *filename, FILE *stream, uint64_t address,
 	       uint64_t stop_point, int magiccount, char *magic)
 {
-  char *buf = (char *) tsk_malloc (sizeof (char) * (string_min + 1));
+  char *buf = (char *) malloc (sizeof (char) * (string_min + 1));
+  if (malloc == NULL) {
+      fprintf(stderr, "Error allocating memory\n");
+      return;
+  }
 
   while (1)
     {
